<?php declare(strict_types=1);
namespace DiviManagerPro\Action;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Action\AbstractAction;
use DiviManager\Service\Profile as ProfileService;
use DiviManager\Service\Site as SiteService;
use DiviManagerPro\Service\Platform as PlatformService;
use DiviManagerPro\Service\PlatformProfile as PlatformProfileService;
/**
 * Perform actions with the platform
 *
 * refresh the platform information for a site
 * clone a site on that platform
 */
class Platform extends AbstractAction
{
    const ACTION_PROVISION = 'platform_provision';
    const ACTION_CLONE     = 'platform_clonesite';
    const ACTION_REFRESH   = 'refreshplatform';

    /**
     * These are the steps during a provision operation
     *  NEW - start a new clone process
     *  DNS - setup the dns information
     *  WP - create the wordpress install
     *  COMPLETE - finished
     */
    const PROVISION_STEP_NEW = 'PROVISION_STEP_NEW';
    const PROVISION_STEP_DNS = 'PROVISION_STEP_DNS';
    const PROVISION_STEP_WP  = 'PROVISION_STEP_WP';
    const PROVISION_STEP_COMPLETE  = 'PROVISION_STEP_COMPLETE';

    /**
     * These are the steps during a clone/migrate operation
     *  NEW - start a new clone process
     *  FILE - copy the files
     *  DB - copy the database
     *  CONFIG - update configuration
     *  COMPLETE - finished
     */
    const CLONE_STEP_NEW = 'CLONE_STEP_NEW';
    const CLONE_STEP_FILE  = 'CLONE_STEP_FILE';
    const CLONE_STEP_DB  = 'CLONE_STEP_DB';
    const CLONE_STEP_CONFIG  = 'CLONE_STEP_CONFIG';
    const CLONE_STEP_COMPLETE  = 'CLONE_STEP_COMPLETE';

    public function __construct()
    {
        /**
         * Setup the supported commands
         * Might be configurable?
         */
        $this->commands = [
            'dbexport' => [
            ],
            'rsync' => [
            ],
        ];
    }

    public function setPlatformService(PlatformService $sm)
    {
        $this->platformService = $sm;
    }

    public function setPlatformProfileService(PlatformProfileService $sm)
    {
        $this->platformProfileService = $sm;
    }

    public function setProfileService(ProfileService $pm)
    {
        $this->profileService = $pm;
    }

    public function setSiteService(SiteService $pm)
    {
        $this->siteService = $pm;
    }

    public function registerActions($worker)
    {
        $worker->actions[self::ACTION_PROVISION] = [$this, 'provisionSite'];
        $worker->actions[self::ACTION_CLONE] = [$this, 'cloneSite'];
        $worker->actions[self::ACTION_REFRESH] = [$this, 'refreshPlatform'];
    }

    public function refreshPlatform($site)
    {
        $platform = $this->platformForSite($site);
        if (null === $platform) {
            return $site;
        }
        $data = $site->getData();
        $data['platform'] = $platform->getSite($site);
        $site->setData($data);

        return $site;
    }

    /**
     * Provision a new install for the site
     *  - create the new site
     */
    public function provisionSite( $site, $args )
    {
        error_log(
            __CLASS__.__FUNCTION__ .
            print_r(['site' => $site, 'args' => $args], true)
        );
        $profile = $this->profileService->getProfile($site->getProfileID());
        $platform = $this->platformProfileService->getPlatformForProfile($profile);

        if (null === $platform) {
            error_log( __CLASS__.__FUNCTION__ . ': No platform for site, skipping clone');
            // no platform, skip the install operation
            return $site;
        }

        $appName = $this->siteService->justDomain($site->getDomain());
        $installID = $platform->provision($appName);
        $this->platformService->addSiteToPlatform($platform, $site, $installID);

        $this->siteService->updateSite($site);

        // add update ssl to queue, hopefully the delay will be enough?
        $this->queueService->addEventForSiteToQueue(self::ACTION_SSL, $site,);

        return $site;
    }
    /**
     * Clone a site
     *  - migrate from the source site to the new site
     */
    public function cloneSite( $site, $args )
    {
        error_log(
            __CLASS__.__FUNCTION__ .
            print_r(['site' => $site, 'args' => $args], true)
        );
        $profile = $this->profileService->getProfile($site->getProfileID());
        $platform = $this->platformService->platform($profile->getPlatformID());

        if (null === $platform) {
            error_log( __CLASS__.__FUNCTION__ . ': No platform for site, skipping clone');
            // no platform, skip the install operation
            return $site;
        }

        $appName = $this->siteService->justDomain($site->getDomain());
        // cloning from a profile or a source site?
        if (isset($args['source'])) {
            // given a source to clone from
            $source = $sm->getSite($args['source']);
            if (null === $source) {
                error_log( __CLASS__.__FUNCTION__ . ': Source does not match an existing site. ');
            } else {
                $installID = $platform->cloneInstall($source->getInstallID(), $appName);
            }
        } else {
            // no source, use the profile
            $installID = $platform->cloneInstall($profile->getSource(), $appName);
        }
        $site->setInstallID($installID);

        $this->siteService->updateSite($site);

        // add update ssl to queue, hopefully the delay will be enough?
        $this->queueService->addEventForSiteToQueue(self::ACTION_SSL, $site,);

        return $site;
    }

    /**
     * Migrate a site
     *
     * Copy all the files and database content from one site into another site
     * This is useful for setting up copies for development
     * Also for cloning
     *
     * Both source and destination sites must already exist
     */
    public function migrateSite($srcSite, $dstSite)
    {
        // ask the src site's worker to the database and content
        // send the database and content to the destination site
        // destination site updates db prefix in wp-config.php
        // destination site updates siteurl and home options
        // destination site flushes rewrites?  this requires wp-cli config values which may not be available
        /*
# migrate a site

# usage:
# sh migrate.sh sourceuser sourceapp dstuser dstapp dsturl

#defaults
php="/usr/bin/php7.4-sp"
wp="/opt/sp/bin/wp-cli.phar"

srcuser=$1
srcapp=$2
dstuser=$3
dstapp=$4
dsturl=$5
srcpath=/srv/users/$srcuser/apps/$srcapp/public
dstpath=/srv/users/$dstuser/apps/$dstapp/public

# capture db prefix
srcprefix=$(sudo -u $srcuser -- $php $wp --path=$srcpath config get --global=table_prefix)
dstprefix=$(sudo -u $dstuser -- $php $wp --path=$dstpath config get --global=table_prefix)

# reset the destination database
sudo -u $dstuser -- $php $wp --path=$dstpath db reset --yes
# export database to file
#exportdb=$(sudo -u $srcuser -- $php $wp --path=$srcpath db export --porcelain)
# import to the destination database
#sudo -u $dstuser -- $php $wp --path=$dstpath db import $exportdb
# export src database to dest database
sudo -u $srcuser -- $php $wp --path=$srcpath db export - | \
    sudo -u $dstuser -- $php $wp --path=$dstpath db import -

# copy the plugins, themes, and uploads folders recursively
sudo cp -aRp $srcpath/wp-content/plugins $dstpath/wp-content/
sudo cp -aRp $srcpath/wp-content/themes $dstpath/wp-content/
sudo cp -aRp $srcpath/wp-content/uploads $dstpath/wp-content/
# correct ownership of the plugins, themes, and uploads folders recursively
sudo chown -R $dstuser:$dstuser $dstpath/wp-content/plugins
sudo chown -R $dstuser:$dstuser $dstpath/wp-content/themes
sudo chown -R $dstuser:$dstuser $dstpath/wp-content/uploads

# fix the db prefix
sudo sed -i "s/$dstprefix/$srcprefix/g" $dstpath/wp-config.php

# search-replace fix strings
sudo -u $dstuser -- $php $wp --path=$dstpath search-replace "$srcapp" "$dstapp"
# fix
sudo -u $dstuser -- $php $wp --path=$dstpath option set siteurl "$dsturl"
sudo -u $dstuser -- $php $wp --path=$dstpath option set home "$dsturl"
# link our custom wp-cli config which enables rewrite
sudo ln -s /opt/wordpress/custom/wp-cli.yml /srv/users/$dstuser/.wp-cli/config.yml
# flush rewrite so the permalinks get updated
sudo -u $dstuser -- $php $wp --path=$dstpath rewrite flush --hard
*/
    }

    /**
     * Migrate source site to destination site
     */
    public function migrate($srcUser, $srcApp, $destUser, $destApp)
    {
        //$sourcePath = $options['sourcePath'];

        $command = '/opt/wordpress/bin/migrate.sh';
        $params = '%srcUser %srcApp %destUser %destApp';

        $args = [
            '%srcUser' => $srcUser,
            '%srcApp' => $srcApp,
            '%destUser' => $destUser,
            '%destApp' => $destApp
        ];

        // put the parameters and arguments together
        $shell = $this->buildShell($command, $params, $args);

        // run the shell
        return $this->doShell($shell);
    }
}
