<?php declare(strict_types=1);
namespace DiviManagerPro;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Registry;

/**
 * Support configuration and admin features for the Serverpilot platform
 */
class Admin
{
    protected $hookPrefix = 'divimanager'; // need our own prefix?  dmpro?
 
    /**
     * @var Registry the core class
     */
    protected $registry;

    /**
     * @var String plugin slug, used as base for options
     */
    protected $slug;

    /**
     * @var Option options manager
     */
    protected $option;

    /**
     * @var boolean is debug enabled?
     */
    protected $debug;

    public function __construct()
    {
        $this->slug = 'divimanager-pro';
    }

    public function setOption($option)
    {   
        $this->option = $option;
    }

    public function setRegistry(Registry $registry)
    {   
        $this->registry = $registry;
    }

    public function setSlug(String $s)
    {
        $this->slug = $s;
    }

    public function setDebug(bool $debug)
    {
        $this->debug = true;
    }

    public function isDebug()
    {
        return true === $this->debug;
    }

    public function init()
    {   
        //add_action( 'admin_menu', [$this, 'admin_menu'] );
        add_action( $this->hookPrefix . '_admin_menu', [$this, 'admin_menu'], 11);
        add_filter( 'divimanager_mysettings', [$this, 'register_mysettings'], 2 );
        add_filter( 'divimanager_options_validate', [$this, 'plugin_options_validate'] );
        add_action( 'admin_enqueue_scripts', [$this, 'enqueue_scripts'] );
    }
    
    public function register_mysettings($settings)
    {
        $optionName = $this->option->getOptionName();
        $sectionName = $this->slug . '_section_pro';
        $settings[$this->slug . '-pro'] = [
            $sectionName,
            'Pro Settings',
            [$this, 'pro_settings'],
            $optionName,
            'settings' => [
                [
                    $this->slug . '_field_license',
                    'License Key',
                    [$this, 'setting_license'],
                    $optionName,
                    $sectionName,
                    [
                        'label_for' => 'divimanager_options[license]'
                    ]
                ],
                [
                    $this->slug . '_field_email',
                    'License Email',
                    [$this, 'setting_email'],
                    $optionName,
                    $sectionName,
                    [
                        'label_for' => 'divimanager_options[email]'
                    ]
                ],
            ]
        ];

        return $settings;
    }

    public function admin_menu ()
    {
        // add the platform pages
        $platformPage  = $this->registry->getService(Page\Platform::class);
        $platformHook = add_submenu_page( 'divimanager-menu', 'Platforms', 'Platforms', 'manage_options', 'divimanager-platform', [$platformPage, 'index']);
        $platformOptionsHook = add_submenu_page( $platformHook, 'Platform Options', 'Platform Options', 'manage_options', 'divimanager-platform-options', [$platformPage, 'options']);
        $platformDetailHook = add_submenu_page( $platformHook, 'Platform Details', 'Platform Details', 'manage_options', 'divimanager-platform-details', [$platformPage, 'details']);
        $platformServerHook = add_submenu_page( $platformHook, 'Platform Servers', 'Platform Servers', 'manage_options', 'divimanager-platform-servers', [$platformPage, 'servers']);
        //$platformImportHook = add_submenu_page( 'divimanager-platform', 'Platform Import', 'Platform Import', 'manage_options', 'divimanager-platform-import', [$platformPage, 'import']);
        add_action( "load-$platformDetailHook", [ $platformPage, 'screen_option' ] );

        // add the server pages
        $serverPage  = $this->registry->getService(Page\Server::class);
        $serverHook = add_submenu_page( 'divimanager-menu', 'Servers', 'Servers', 'manage_options', 'divimanager-server', [$serverPage, 'index']);
        $serverOptionsHook = add_submenu_page( $platformHook, 'Platform Options', 'Platform Options', 'manage_options', 'divimanager-server-details', [$serverPage, 'options']);
        add_action( "load-$platformDetailHook", [ $platformPage, 'screen_option' ] );

        // this is a simple implementation for setting option values on managed site                                       
        //$optionPage = $this->registry->getService(Page\Option::class);                                                   
        //$optionHook = add_submenu_page( 'divimanager-menu', 'Site Options', 'Site Options', 'manage_options', 'divimanager-site-option', [$optionPage, 'option']);
    }

    /**
     * Enqueue javascript for pages
     *
     * https://developer.wordpress.org/plugins/javascript/enqueuing/
     */
    public function enqueue_scripts($hook)
    {
        if ('manager_page_divimanager-server' === $hook ||
            'admin_page_divimanager-platform-details' === $hook) {
            \wp_enqueue_script(
                'ajax-script',
                \plugins_url( '/js/server.js', $this->registry->getService(DiviManagerPro::class)->getMainPluginFile() ),
                [ 'jquery' ],
                '1.0.0',
                [
                   'in_footer' => true,
                ]
            );

            \wp_localize_script(
                'ajax-script',
                'my_ajax_obj',
                [
                    'ajax_url' => '/?rest_route=/dm/v3',
                    'nonce'    => \wp_create_nonce( 'wp_rest' ),
                    //'nonce'    => \wp_create_nonce( 'title_example' ),
                ]
            );
        }
    }

    /**
     * custom plugin settings
     */
    public function pro_settings($args)
    {
        echo '<p id="' . esc_attr( $args['id'] ) . '">All the settings which the DiviManager-Pro version will use.</p>';
    }

    public function setting_license($args)
    {
        $license = $this->option->getOption('license');
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="password" value="' . esc_attr ( $license ) . '" />';
    }

    public function setting_email($args)
    {
        $email = $this->option->getOption('email');
        echo '<input id="' . esc_attr( $args['label_for'] ) . '" name="' . esc_attr( $args['label_for'] ) . '" type="text" value="' . esc_attr ( $email ) . '" />';
    }

    public function plugin_options_validate($input)
    {
        $input['license'] = \sanitize_text_field($input['license']);
        $email = \sanitize_text_field($input['email']);
        if (is_email($email)) {
            $input['email'] = $email;
        } else {
            $input['email'] = '';
        }

        return $input;
    }
}
