<?php declare(strict_types=1);
namespace DiviManagerPro\Command;

use DiviManager\Service\Site as SiteService;
use DiviManagerPro\Action\Migrate as MigrateAction;

/**
 * Migrate a site, from src site to dest site
 */
class Migrate extends \WP_CLI_Command
{
    /**
     * @var SiteService
     */
    protected $siteService;

    /**
     * @var MigrateAction
     */
    protected $migrateAction;

    public function setSiteService(SiteService $sm)
    {
        $this->siteService = $sm;
    }

    public function setMigrateAction(MigrateAction $a)
    {
        $this->migrateAction = $a;
    }

    /**
     * Get export information from src site
     *
     * ## OPTIONS
     *
     * <src_site_id>
     * : The id of the site which will be the source of the data to clone.
     *
     * [--dry-run]
     * : Just print out the steps, do not do them.  True by default
     *
     * ## EXAMPLES
     *
     *     wp manager migrate export <src_site_id> [--dry-run=<dry-run>]
     *
     * @subcommand export src_site_id
     */
    public function export($args, $assoc_args )
    {
        $srcSiteID = $args[0];
        $dryrun = isset($assoc_args['dry-run']) ? $assoc_args['dry-run'] : true;

        $site = $this->siteService->getSite($srcSiteID);
        if (null === $site) {
            \WP_CLI::error( 'Source site not found' );
        }

        $this->migrateAction->setDryrun($dryrun);
        $exportData = $this->migrateAction->exportFromSite($site);

        \WP_CLI::success( 'Success' );
    }

    /**
     * Get export status for src site
     *
     * ## OPTIONS
     *
     * <src_site_id>
     * : The id of the site which will be the source of the data to clone.
     *
     * ## EXAMPLES
     *
     *     wp manager migrate export-status <src_site_id>
     *
     * @subcommand export-status src_site_id
     */
    public function exportStatus($args, $assoc_args )
    {
        $srcSiteID = $args[0];

        $site = $this->siteService->getSite($srcSiteID);
        if (null === $site) {
            \WP_CLI::error( 'Source site not found' );
        }

        $siteData = $site->getData();
        if (! isset($siteData['exportData']) || empty($siteData['exportData'])) {
            \WP_CLI::success( 'Export has not been started on the site: ' . $srcSiteID);
        } else {
            $table = [
                [
                    'key' => 'status',
                    'value' => isset($siteData['exportData']['status']['status']) ? $siteData['exportData']['status']['status'] : 'Not started',
                ],
                [
                    'key' => 'wpcontent',
                    'value' => isset($siteData['exportData']['status']['wpcontent']) ? $siteData['exportData']['status']['wpcontent'] : 'No',
                ],
                [
                    'key' => 'database',
                    'value' => isset($siteData['exportData']['status']['database']) ? $siteData['exportData']['status']['database'] : 'No',
                ],
            ];
            \WP_CLI\Utils\format_items( 'table', $table, [ 'key', 'value' ] );
        }
        var_dump($siteData['exportData']);
    }

    /**
     * Fetch an export file from a site
     *
     * ## OPTIONS
     *
     * <src_site_id>
     * : The id of the site which will be the source of the data to clone.
     *
     * [--file=<file>]
     * : The export file to download.  Either the wpcontent or database archive.
     * ---
     * default: wpcontent
     * options:
     *   - wpcontent
     *   - database
     * ---
     *
     * [--dry-run]
     * : Just print out the steps, do not do them.  True by default
     *
     * ## EXAMPLES
     *
     *     wp manager migrate fetch <src_site_id> [--file=<file>] [--dry-run=<dry-run>]
     *
     * @subcommand fetch src_site_id file
     */
    public function fetch($args, $assoc_args )
    {
        $srcSiteID = $args[0];
        $file = isset($assoc_args['file']) ? $assoc_args['file'] : 'database';
        $dryrun = isset($assoc_args['dry-run']) ? $assoc_args['dry-run'] : true;

        $site = $this->siteService->getSite($srcSiteID);
        if (null === $site) {
            \WP_CLI::error( 'Source site not found' );
        }

        $siteData = $site->getData();
        if (! isset($siteData['exportData'])) {
            $exportData = [];
        } else {
            $exportData = $siteData['exportData'];
        }

        $this->migrateAction->setDryrun($dryrun);
        $data = $this->migrateAction->fetchFromSite($site, $file);
        \WP_CLI::success( 'Success' );
    }

    /**
     * Import a file to a new site
     *
     * ## OPTIONS
     *
     * <src_site_id>
     * : The id of the source site.
     *
     * <dst_site_id>
     * : The id of the destination site.
     *
     * [--file=<file>]
     * : The export file to download.  Either the wpcontent or database archive.
     * ---
     * default: wpcontent
     * options:
     *   - wpcontent
     *   - database
     * ---
     *
     * [--dry-run]
     * : Just print out the steps, do not do them.  True by default
     *
     * ## EXAMPLES
     *
     *     wp manager migrate import-start <src_site_id> <dst_site_id> [--file=<file>] [--dry-run=<dry-run>]
     *
     * @subcommand import-start src_site_id dst_site_id [--file=<file>] [--dry-run=<dry-run>]
     */
    public function importStart($args, $assoc_args )
    {
        $srcSiteID = $args[0];
        $dstSiteID = $args[1];
        $file = isset($assoc_args['file']) ? $assoc_args['file'] : 'wpcontent';
        $dryrun = isset($assoc_args['dry-run']) ? $assoc_args['dry-run'] : true;

        $site = $this->siteService->getSite($srcSiteID);
        if (null === $site) {
            \WP_CLI::error( 'Source site not found' );
        }

        $dstSite = $this->siteService->getSite($dstSiteID);
        if (null === $dstSite) {
            \WP_CLI::error( 'Destination site not found' );
        }

        $this->migrateAction->setDryrun($dryrun);
        $this->migrateAction->importToSite($site, $dstSite, $file);
        \WP_CLI::success( 'Success' );
    }

    /**
     * Get import status for dst site
     *
     * ## OPTIONS
     *
     * <dst_site_id>
     * : The id of the site which will be the source of the data to clone.
     *
     * ## EXAMPLES
     *
     *     wp manager migrate import-status <dst_site_id>
     *
     * @subcommand import-status dst_site_id
     */
    public function importStatus($args, $assoc_args )
    {
        $dstSiteID = $args[0];

        $site = $this->siteService->getSite($dstSiteID);
        if (null === $site) {
            \WP_CLI::error( 'Destination site not found' );
        }

        $importData = $this->migrateAction->importStatusFromSite($site);

        if (empty($importData)) {
            \WP_CLI::success( 'Import has not been started on the site: ' . $dstSiteID);
        } else {
            $table = [
                [
                    'key' => 'status',
                    'value' => $importData['status'],
                ],
                [
                    'key' => 'wpcontent',
                    'value' => false == $importData['wpcontent'] ? 'No' : $importData['wpcontent'],
                ],
                [
                    'key' => 'database',
                    'value' => false == $importData['database'] ? 'No' : $importData['database'],
                ],
            ];
            \WP_CLI\Utils\format_items( 'table', $table, [ 'key', 'value' ] );
        }

        \WP_CLI::success( 'Success' );
    }

    /**
     * Starts the migration process from a source site to a destination site
     *
     * ## OPTIONS
     *
     * <src_site_id>
     * : The id of the site which will be the source of the data to clone.
     *
     * <dst_site_id>
     * : The id of the site which is the destination of the clone.
     *
     * [--dry-run]
     * : Just print out the steps, do not do them.  True by default
     *
     *
     * ## EXAMPLES
     *
     *     wp manager clone <src_site_id> <dst_site_id> [--dry-run=<dry-run>]
     *
     * @subcommand migrate
     */
    public function migrate($args, $assoc_args )
    {
        $srcSiteID = $args[0];
        $dstSiteID = $args[1];
        $dryrun = isset($assoc_args['dry-run']) ? $assoc_args['dry-run'] : true;

        $srcSite = $this->siteService->getSite($srcSiteID);
        if (null === $srcSite) {
            \WP_CLI::error( 'Source site not found' );
        }

        $dstSite = $this->siteService->getSite($dstSiteID);
        if (null === $dstSite) {
            \WP_CLI::error( 'Destination site not found' );
        }

        //\WP_CLI::log( print_r($srcSiteID, true) );
        //\WP_CLI::log( print_r($dstSiteID, true) );
        //\WP_CLI::log( print_r($dryrun, true) );

        $this->migrateAction->setDryrun($dryrun);
        $exportData = $this->migrateAction->exportFromSite($srcSite);
        $wpcontent = $this->migrateAction->fetchFromSite($srcSite, 'wpcontent');
        $database = $this->migrateAction->fetchFromSite($srcSite, 'database');
        $exportData = $this->migrateAction->importToSite($dstSite, $srcSite);
        \WP_CLI::success( 'Success' );
    }

    /**
     * Test pushing a file via post
     *
     * ## OPTIONS
     *
     * <src_site_id>
     * : The id of the site which will be the source of the data to clone.
     *
     * <dst_site_id>
     * : The id of the site which is the destination of the clone.
     *
     * [--dry-run]
     * : Just print out the steps, do not do them.  True by default
     *
     *
     * ## EXAMPLES
     *
     *     wp manager migrate import-push <src_site_id> <dst_site_id> [--dry-run=<dry-run>]
     *
     * @subcommand import-push
    public function importPush($args, $assoc_args )
    {
        $srcSiteID = $args[0];
        $dstSiteID = $args[1];
        $dryrun = isset($assoc_args['dry-run']) ? $assoc_args['dry-run'] : true;

        $srcSite = $this->siteService->getSite($srcSiteID);
        if (null === $srcSite) {
            \WP_CLI::error( 'Source site not found' );
        }

        $dstSite = $this->siteService->getSite($dstSiteID);
        if (null === $dstSite) {
            \WP_CLI::error( 'Destination site not found' );
        }

        $response = $this->migrateAction->importPushToSite($dstSite, $srcSite, 'wpcontent');
        var_dump($response);
        \WP_CLI::success( 'Success' );
    }
     */
}
