<?php declare(strict_types=1);
namespace DiviManagerPro;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Registry;
use DiviManager\Service\Worker;
use DiviManager\Helper;
use DiviManager\Helper\Option;
use DiviManagerPro\Reporter\Reporter;
use DiviManagerPro\Serverpilot\Serverpilot;
use DiviManagerPro\Woocommerce\Woocommerce;
use DiviManagerPro\Api;
use ElLib\ElLib;

class DiviManagerPro
{
    public const VERSION = '0.0.9';
    public const VERSION_NAME = 'divimanager_pro_db_version';
    public const OPTIONS_NAME = 'divimanager_pro_options';
	public const SLUG = 'divimanager-pro';
	public const ELLIB_KEY = 'CD8E3E517519100B';
	public const ELLIB_PRODUCT_ID = '2';
	public const ELLIB_SLUG = 'divimanager/divimanager-pro';
    public const ELLIB_URL = 'https://www.divimanager.com/?rest_route=/licensor/';
    public const BASE = 'dm/v3';


    protected $hookPrefix = 'divimanager'; // need our own prefix?  dmpro?

    /**
     * var DiviManagerPro
     *
     * Our instance for the singleton pattern
     */
    static $instance = false;

	/**
     * var string
     *  
     * Our main plugin file
     */     
    protected $mainPluginFile;

    public function __construct($mainPluginFile)
    {
        if ($this->checkDiviManagerActive()) {
            $this->mainPluginFile = $mainPluginFile;

            register_activation_hook( $mainPluginFile, [$this, 'activation'] );
            register_deactivation_hook( $mainPluginFile, [$this, 'deactivation'] );
            // uninstall needs to be a public static function?
            register_uninstall_hook( $mainPluginFile, [static::class, 'uninstall'] );

            // early in the load order, 
            add_action( 'plugins_loaded', [$this, 'plugins_loaded']);
            // when wordpress is initialized
            add_action( 'init', [$this, 'wp_init'], 11);
            // called when loading an admin page, pages need to be already setup for the menus though
            //add_action( 'admin_init', [$this, 'admin_init'], 11);
            // called when registering services in the registry
            add_action( $this->hookPrefix . '_services_init', [$this, 'services_init'], 11);

            $this->reporter = new Reporter($this->hookPrefix);
            $this->serverpilot = new Serverpilot($this->hookPrefix);
            $this->woocommerce = new Woocommerce($this->hookPrefix, $this);
            //$this->woocommerce = new Woocommerce();
            //$this->serverpilot = new ServerPilot();
            //$this->dnsmadeeasy = new DnsMadeEasy();
        } else {
            require_once (ABSPATH . 'wp-admin/includes/plugin.php');
            \deactivate_plugins($mainPluginFile);
        }
    }

	public function getMainPluginFile()
	{
		return $this->mainPluginFile;
	}

    /**
     * Wrap checks for an active plugin
     */
    public function checkDiviManagerActive()
    {
        return in_array( 'divimanager/divimanager.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) );
    }

    /**
     * Run when the plugin is activated
     *
     * This is usually run after initial installation
     * Handle initial setup
     *
     * These are tasks which need to happen once, when the plugin is first activated
     */
    public function activation()
    {
        $this->divimanager = \DiviManager\DiviManager::getInstance(); // get the main plugin's instance for use
        $this->registry = $this->divimanager->getRegistry();
        $this->divimanager->services_init($this->registry); // we need to initialize all the services from the main plugin when activated, probably need the activation changed to not require external services be available
        $this->registry->getService(Service\Platform::class)->activation();
        $this->registry->getService(Service\Server::class)->activation();
        $this->registry->getService(Service\PlatformServer::class)->activation();
        $this->install();
    }

    /**
     * Runs when the plugin is deactivated
     *
     * Removes scheduled events
     */
    public function deactivation()
    {
        // delete the option, set to null?
        delete_option( self::VERSION_NAME, NULL );
        update_option( self::VERSION_NAME, NULL, FALSE );
    }

    /**
     * Runs when the plugin has been first deactivated and then deleted
     *
     * Removes all data related to the plugin
     */
    public static function uninstall ()
    {
        // delete the option, set to null?
        delete_option( self::VERSION_NAME, NULL );
        update_option( self::VERSION_NAME, NULL, FALSE );
        delete_option( self::OPTIONS_NAME, NULL );
        update_option( self::OPTIONS_NAME, NULL, FALSE );
    }

    /**
     * Runs when the plugins_loaded action is called
     * Use this to do db updates
     *
     * Run actions which need to occur every page request, but before wordpress init
     */
    public function plugins_loaded()
    {
        $this->divimanager = \DiviManager\DiviManager::getInstance(); // get the main plugin's instance for use
        $this->registry = $this->divimanager->getRegistry();
    }

    /**
     * Runs when the init action is called, after plugins are loaded
     */
    public function wp_init()
    {
        $this->update_db_check();
        add_filter( 'query_vars', [$this, 'addQueryVars'] );

        register_activation_hook($this->mainPluginFile, [$this, 'activation']);
        register_deactivation_hook($this->mainPluginFile, [$this, 'deactivation']);

        // load our options
        $options = get_option(self::OPTIONS_NAME);

        // add wp-cli command
        if ( defined( 'WP_CLI' ) && WP_CLI ) {
            //$this->registry->setService(Command\CloneCommand::class, Command\CloneCommandFactory::class);
            //$cloneCommand = $this->registry->getService(Command\CloneCommand::class);
            //\WP_CLI::add_command( 'manager clone', [$cloneCommand, 'cloneSite'] );
            $this->registry->setService(Command\Migrate::class, Command\MigrateFactory::class);
            $migrateCommand = $this->registry->getService(Command\Migrate::class);
            \WP_CLI::add_command( 'manager migrate', $migrateCommand );
        }

        if (is_admin() || (defined('WP_CLI') && WP_CLI)) {
            $this->admin_init();
			$this->doLicenseCheck();
        }

        // rest api, even authenticated via application password, does not set admin, so is_admin will fail
        // it is only admin when cookies are used.
        // at this point the user is not available either
        //if (current_user_can('administrator')) {

        $api = new Api();
        $api->setManager($this);
        $api->setRegistry($this->registry);
        $api->setBase(self::BASE);
        add_action( 'rest_api_init', [$api, 'register_rest_routes']);
        ////}
    }

    /**
     * Runs when the admin_init action is called, admin pages are being viewed
     *
     * No need to use an is_admin() check as this is only run before loading admin pages
     */
    public function admin_init()
    {
        $this->registry->getService(Admin::class)->init();
        \do_action ($this->hookPrefix . '_admin_init', $this->registry);
    }

    /**
     * Setup our services.
     *
     * Every plugin will be adding their own
     */
    public function services_init(Registry $registry)
    {
        $registry->setService(__CLASS__, $this); 

        $registry->setService(Table\PlatformInstall::class, Table\PlatformInstallFactory::class);
        $registry->setService(Table\PlatformServer::class, Table\PlatformServerFactory::class);
        $registry->setService(Service\Platform::class, Service\PlatformFactory::class);
        $registry->setService(Service\Server::class, Service\ServerFactory::class);
        $registry->setService(Service\PlatformServer::class, Service\PlatformServerFactory::class);
        $registry->setService(Service\PlatformProfile::class, Service\PlatformProfileFactory::class);
        // pages
        $registry->setService(Page\Platform::class, Page\PlatformFactory::class);
        $registry->setService(Page\Server::class, Page\ServerFactory::class);
        // override the free create and clone site pages
        $registry->setService(\DiviManager\Page\Create::class, Page\CreateFactory::class);
        $registry->setService(\DiviManager\Page\CloneSite::class, Page\CloneSiteFactory::class);
        // override profile page
        $registry->setService(\DiviManager\Page\Profile::class, Page\ProfileFactory::class);

        $registry->setService(Action\Core::class, Action\CoreFactory::class);
        $registry->setService(Action\Platform::class, Action\PlatformFactory::class);
        $registry->setService(Action\Migrate::class, Action\MigrateFactory::class);

        $registry->setService(Admin::class, AdminFactory::class);

        // add our action to the worker service
        $worker = $registry->getService(Worker::class);
        $worker->addAction($registry->getService(Action\Core::class));
        $worker->addAction($registry->getService(Action\Platform::class));
        $worker->addAction($registry->getService(Action\Migrate::class));
	}

    /**
     * install db schema
     *
     * detects the version of the schema
     * if different, runs the script
     */
    protected function install ()
    {
        $installed_ver = get_option( self::VERSION_NAME );

        if ( $installed_ver != self::VERSION ) {
            // set some defaults
            if ($installed_ver < '0.0.6') {
            }

            if ($installed_ver < '0.0.7') {
                $this->update007();
            }

            if ($installed_ver < '0.0.10') {
                $this->update0010();
            }
        }

        update_option( self::VERSION_NAME, self::VERSION );
    }

    /**
     * Prior to version 0.0.6, db update
     */
    protected function update006()
    {
    }

    /**
     * Update step from 0.0.6 to 0.0.7
     */
    protected function update007()
    {
        // remove old version option
        delete_option( 'dmpro_db_version', NULL );
        // remove old options
        delete_option( 'dmp_options', NULL );

        $this->registry->getService(Service\Server::class)->install();
        $this->registry->getService(Service\PlatformServer::class)->install();
    }

    /**
     * Update step from 0.0.9 to 0.0.10
     */
    protected function update0010()
    {
        $this->registry->getService(Service\PlatformProfile::class)->install();
    }

    /**
     * Check if our version matches the version in the database
     * If not, we need to update.
     */
    public function update_db_check()
    {
        if ( get_site_option( self::VERSION_NAME ) != self::VERSION ) {
            $this->install();
        }
    }

    /**
     * Need to let wordpress know what query variables we use
     */
    public function addQueryVars( $vars )
    {
        //$vars[] = 'site_id';
        //$vars[] = 'slug';

        return $vars;
    }

    /**
     * End setup/hooks
     */

    /**
     * Check our license and for updates
     */
    public function doLicenseCheck()
    {
        $this->option = $this->registry->getService(Option::class);
        // license support
        $licenseKey = $this->option->getOption('license');
        $licenseEmail = $this->option->getOption('email');
        $this->elLib = new ElLib(
            self::ELLIB_KEY,
            self::ELLIB_PRODUCT_ID,
            self::ELLIB_SLUG,
            self::ELLIB_URL,
            $licenseKey,
            $licenseEmail,
            $this->mainPluginFile
        );
        $this->elLib->addOnDelete([$this, 'deleteLicenseKey']);
        if ($this->elLib->CheckWPPlugin($error, $responseObj)) {
            if (true === $responseObj->is_valid) {
            }
        }
    }

    /**
     * License support
     */
    public function deleteLicenseKey()
    {
        $this->option->setOption('license', null);
    }

    /**
     * Whether the Woocommerce functionality is enabled
     *
     * @return bool
     */
    public function isWoocommerceEnabled()
    {
        return true === $this->option->getOption('woocommerce_enabled');
    }

    /**
     * Whether the DnsMadeEasy functionality is enabled
     *
     * @return bool
     */
    public function isDmeEnabled()
    {
        return true === $this->option->getOption('dme_enabled');
    }

    /**
     * create a new site
     * Used by ecommerce extensions
     * also can be manually triggered to add new sites without orders
     * Can clone an existing site into the new site
     */
    public function newSite($url, $profile, $user, $subscription, $note, $staging = true, $provision = false, $clone = false, $refresh = true)
    {
        error_log(
            __CLASS__.':'.__FUNCTION__ .
            print_r( ['url' => $url, 'profile' => $profile, 'provision' => true === $provision, 'clone' => true === $clone], true)
        );

        $this->profileService = $this->registry->getService(\DiviManager\Service\Profile::class);
        $this->queueService = $this->registry->getService(\DiviManager\Service\Queue::class);
        $this->platformService = $this->registry->getService(\DiviManagerPro\Service\Platform::class);
        $this->platformProfileService = $this->registry->getService(\DiviManagerPro\Service\PlatformProfile::class);

        // make the new site entry in manager, using the core divimanager logic
        $site = $this->divimanager->newSite($url, $profile, $user, $subscription, $note, $staging, false);

        $profile = $this->profileService->getProfile($site->getProfileID());
        $platform = $this->platformProfileService->getPlatformForProfile($profile);

        //if (true === $provision && true == $profile->doesProvision())
        if (true === $provision) {
            if (true === $platform->doesProvision()) {
                $this->queueService->addEventForSiteToQueue(\DiviManagerPro\Action\Platform::ACTION_PROVISION, $site);
                error_log( __CLASS__.':'.__FUNCTION__ . ' Added provision to queue');
                Helper::adminMessage( __( 'Site added to queue for provisioning.  Please allow a few minutes to complete.'), Helper::SUCCESS);
            } else {
                error_log( __CLASS__.':'.__FUNCTION__ . ' No provision');
                Helper::adminMessage( __( 'Profile does not support provisioning a new site on the platform.'), Helper::ERROR);
            }
        }

        //if (true === $clone && true == $profile->doesClone())
        // clone currently controls whether a site is provision on the platform at all
        if (true === $clone) {
            // send this to the queue
            $this->queueService->addEventForSiteToQueue(\DiviManagerPro\Action\Platform::ACTION_CLONE, $site);
            error_log( __CLASS__.':'.__FUNCTION__ . ' Added clone to queue');
            Helper::adminMessage( __( 'Site added to queue for cloning.  Please allow a few minutes to complete'), Helper::SUCCESS);
        }

        if (true === $refresh) {
            $this->queueService->addEventForSiteToQueue(\DiviManager\Action\Site::ACTION_REFRESH, $site);
        }

        return $site;
    }

    /**
     * clone an existing site in a new site
     */
    public function cloneSite($srcSite, $newSite, $refresh = true)
    {
        error_log(
            __CLASS__.':'.__FUNCTION__ .
            print_r( ['url' => $newSite->getUrl(), 'srcSite' => $srcSite], true)
        );

	    if (! is_object($srcSite)) {
            $srcSite = $this->siteService->getSite($srcSite);
        }

        //if (null !== $srcSite) {
            //$site->setProfileId($srcSite->getProfileId());
        //}

        //$site = $this->siteService->createSite($site);

        if (true === $profile->doesClone()) {
            // send this to the queue
            $this->queueService->addEventForSiteToQueue(Action\Site::ACTION_CLONE, $newSite, ['source' => $srcSite->getId()]);
            error_log( __CLASS__.':'.__FUNCTION__ . ' Added clone to queue');
            Helper::adminMessage( __( 'Site added to queue for cloning.  Please allow a few minutes to complete'), Helper::SUCCESS);
        } else {
            error_log( __CLASS__.':'.__FUNCTION__ . ' Profile does not support cloning');
        }

        if (true === $refresh) {
            $this->queueService->addEventForSiteToQueue(Action\Site::ACTION_REFRESH, $site);
        }

        return $site;
    }
}
