<?php declare(strict_types=1);
namespace DiviManager\Page;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\DiviManager;
use DiviManager\Service\Site as SiteService;
use DiviManager\Service\Profile as ProfileService;
use DiviManager\Model\Site as SiteModel;
use DiviManager\Action\Site as SiteAction;
use DiviManager\Helper;

/**
 * Page for creating a clone of an existing site
 */
class CloneSite
{
    /**
     * @var DiviManager $manager
     */
    protected $manager;

    /**
     * @var SiteService
     */
    protected $siteService;

    /**
     * @var ProfileService
     */
    protected $profileService;

    public function __construct()
    {
    }

    /**
     * Set the main manager class
     */
    public function setManager(DiviManager $manager)
    {
        $this->manager = $manager;
    }

    public function setSiteService(SiteService $sm)
    {
        $this->siteService = $sm;
    }

    public function setProfileService(ProfileService $qm)
    {
        $this->profileService = $qm;
    }

    /**
     * Handle all our setup for wordpress
     */
    public function init()
    {
        add_action( 'load-divimanager_page_divimanager-site-new', [$this, 'add_headers'] );
    }

    /**
     * Add our headers to the admin header
     */
    public function add_headers()
    {
        get_current_screen()->add_help_tab([
            'id'      => 'main',
            'title'   => __( 'Adding a new site Help' ),
            'content' => '<p>' . __( 'Add some help here' ) . '</p>',
        ]);
    }

    public function makeProfileSelect($profileID = null)
    {
        $html =<<<EOHTML
<select name="profile" id="profile">
    <option value=""></option>
EOHTML;

        $option =<<<EOHTML
    <option value="%s"%s>%s</option>
EOHTML;
        foreach ($this->profileService->getProfiles() as $profile) {
            $selected = '';
            if (null !== $profileID && $profileID == $profile->getId()) {
                $selected = ' selected="selected"';
            }
            $html .= sprintf($option, $profile->getId(), $selected, $profile->getName() . ($profile->doesClone() ? ' ( Clones )' : ''));
        }

        $html .=<<<EOHTML
</select>
EOHTML;
        return $html;
    }

    /**
     * Support creating a clone of an existing site
     */
    public function clone()
    {
        // is this access check necessary?
        if ( !current_user_can( 'manage_options' ) )  {
            wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
        }

        $site_id = $_GET['site_id'];
        if (null === $site_id) {
            // missing site_id, can not clone
            wp_die( __( 'Missing site, can not clone.' ) );
        }
        $srcSite = $this->siteService->getSite($site_id);
        if (null === $srcSite) {
            // missing site, can not clone
            wp_die( __( 'Site not found, can not clone.' ) );
        }

        if (isset($_POST['clone']) && "Clone" === $_POST['clone']) {
            $sanitized = sanitize_text_field($_POST['url']);
            $sanitized = $this->siteService->stripSchemeFromUrl($sanitized);

            if (false === strpos($sanitized, '.')) {
                $sanitized .= '.com'; // add '.com' if no domain extension is provided
            }
            if ($this->siteService->siteExists($sanitized, SiteService::JUST_MANAGER)) {
                Helper::adminMessage(sprintf( __( '%s already exists, please try a different domain name'), $sanitized ), Helper::ERROR);
            } else {
                $profile = sanitize_text_field($_POST['profile']);
                $user = sanitize_text_field($_POST['user']);
                $subscription = sanitize_text_field($_POST['subscription']);
                $note = sanitize_text_field($_POST['note']);
                //$this->siteService->createSite($site);

                if (isset($_POST['staging'])&& 'true' == $_POST['staging']) {
                    $staging = true;
                } else {
                    $staging = false;
                }

                if (isset($_POST['doit']) && 'doit' === $_POST['doit']) {
                    $clone = true;
                } else {
                    $clone = false;
                }

                // tell the manager we want to add a new site
                // the manager will handle the work of connecting to the platform and doing a new install or clone operation
                $site = $this->manager->newSite($sanitized, $profile, $user, $subscription, $note, $staging, false, false);

                if (true === $clone) {
                    $site = $this->manager->cloneSite($srcSite, $newSite, true);
                }

                $managerUrl = admin_url( 'admin.php?page=divimanager-menu' );
                $detailUrl = admin_url( 'admin.php?page=divimanager-site-detail&site_id=' . $site->getId() );
                echo <<<EOHTML
<p>Click to return to <a href="$managerUrl">Manager</a> or to view site <a href="$detailUrl">details</a>.</p>
EOHTML;

                return;
            }
        }

        $customer = $srcSite->getUserID() === null ? null : $srcSite->getUserID();
        $userSelect = \wp_dropdown_users(['echo' => false, 'selected' => $customer, 'show_option_none' => 'None']);
        $profileSelect = $this->makeProfileSelect($srcSite->getProfileID());
?>
    <div class="wrap">
        <h1 class="wp-heading-inline"><?php echo esc_html( get_admin_page_title() ); ?></h1>
        <div id="post-body" class="metabox-holder columns-2">
            <div id="post-body-content">
                <p>Creating a clone of: <?php echo $this->siteService->liveUrlForSite($srcSite); ?></p>
                <div class="meta-box-sortables ui-sortable">
                    <form method="POST">
                    <table class='form-table'>
                    <tr><th>Name</th><td><input type="text" name="name" value="<?php echo $srcSite->getName(); ?>" /></td><td>The name of the site to manage.  It can be the domain or some other label which will identify this site when listing sites.</td></tr>
                    <tr><th>Domain</th><td><input type="text" name="url" value="<?php echo $srcSite->getDomain(); ?>" /></td><td>This is the domain for the website.  Manager uses this to generate the urls to communicate with the site.  And also for registering the staging domain and making staging urls.</td></tr>
                    <tr><th>Subscription ID</th><td><input type="text" name="subscription" value="<?php echo $srcSite->getOrderID(); ?>" /></td><td>This is woocommerce subscription ID for the site.  The subscription ID is different from individual order IDs.</td></tr>
                    <tr><th>Customer</th><td><?php echo $userSelect; ?></td><td>This is customer of the site.</td></tr>
                    <tr><th>Profile</th><td><?php echo $profileSelect; ?></td><td>Select a profile for this site.</td></tr>
                    <tr><th>Note</th><td><textarea name="note"><?php echo $srcSite->getNote(); ?></textarea></td><td>Any note related to the site.</td></tr>
                    <tr><th>Staging</th><td><input type="checkbox" name="staging" value="true" checked \></td><td>Is the site in staging or production?</td></tr>
                    <tr><th>CREATE INSTALL</th><td><input type="checkbox" name="doit" value="doit" \></td><td>Should this site be created on the platform?  If unchecked, the site is only added to Manager.  If checked, Manager will atempt to create the site, using the selected profile.</td></tr>
                    </table>
                    <button type="submit" name="clone" value="Clone">Clone</button>
                    <button type="reset" name="reset" value="Reset">Reset</button>
                    </form>
                </div>
            </div>
        </div>
        <br class="clear">
	</div>
<?php
    }
}
