<?php declare(strict_types=1);
namespace DiviManagerPro\Page;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Page\Profile as ProfilePage;
use DiviManager\Service\Profile as ProfileService;
use DiviManager\Service\Theme as ThemeService;
use DiviManager\Service\Plugin as PluginService;
use DiviManager\Service\Site as SiteService;
use DiviManager\Service\ProfilePlugin as ProfilePluginService;
use DiviManagerPro\Service\Platform as PlatformService;
use DiviManagerPro\Service\PlatformProfile as PlatformProfileService;
use DiviManager\Helper;

/**
 * Manage profiles
 */
class Profile extends ProfilePage
{
    /**
     * @var PlatformService manager for interacting with platforms
     */
    protected $platformService;

    /**
     * @var PlatformProfileService manager for interacting with platform profile joins
     */
    protected $platformProfileService;

    public function setPlatformService(PlatformService $pm)
    {
        $this->platformService = $pm;
    }

    public function setPlatformProfileService(PlatformProfileService $s)
    {
        $this->platformProfileService = $s;
    }

    /**
     * Main page to manage profiles
     */
    public function manage ()
    {
        $addSiteLink =  ' <a href="' . esc_url( admin_url( 'admin.php?page=divimanager-profile-create' ) ) . '" class="page-title-action">Add New</a>';
?>
    <div class="wrap">
        <h1 class="wp-heading-inline"><?php echo esc_html( get_admin_page_title() ); ?></h1>
<?php echo $addSiteLink; ?>
<?php
        $profiles = $this->profileService->getProfiles();
        $count = count($profiles);

        echo <<<EOHTML
        <p>There are $count profiles currently registered with the manager.</p>
        <table>
        <thead>
            <tr><th>Profile</th><th>Description</th></tr>
        </thead>
        <tbody>
EOHTML;
        $row =<<<EOHTML
            <tr><td>%s</td><td>%s</td></tr>
EOHTML;
        $viewLink = '<a href="%s">%s</a>';
        foreach ($profiles as $profile) {
            $viewUrl = admin_url( 'admin.php?page=divimanager-profile-view&profile_id=' . $profile->getId() );
            $viewLinkUrl = sprintf($viewLink, $viewUrl, $profile->getName());
            echo sprintf($row, $viewLinkUrl, $profile->getDescription());
        }
?>
        </tbody>
        </table>
        <br class="clear">
    </div>
<?php
    }

    /**
     * View details of a profile
     */
    public function view ()
    {
        if (! isset($_GET['profile_id']) || empty($_GET['profile_id'])) {
            echo '<p>Profile not specified.  Please return to the profile list and try selecting a profile.</p>';
            return;
        }

        $profileID = $_GET['profile_id'];
        $profile = $this->profileService->getProfile($profileID);

        if (null === $profile) {
            echo '<p>Profile not found.</p>';
            return;
        }

        $editUrl = admin_url( 'admin.php?page=divimanager-profile-update&profile_id=' . $profileID );

        $theme = $this->themeService->getTheme($profile->getThemeID());
        if (null === $theme) {
            $themeName = 'N/A';
        } else {
            $themeUrl = $this->themeService->getUrlForTheme($theme);
            $themeName = sprintf('<a href="%s">%s</a>', $themeUrl, $theme->getName());
        }

        $pluginBlock = '';

        $plugins = $this->profilePluginService->getPluginsForProfile($profile);
        $pluginRow = '<li><a href="%s">%s</a></li>';
        if (null === $plugins || empty($plugins)) {
            $pluginBlock = '<li>No plugins</li>';
        } else {
            foreach ($plugins as $plugin) {
                if (null === $plugin) {
                    $pluginBlock .= '<li>Plugin not found</li>';
                } else {
                    $pluginUrl = $this->pluginService->getUrlForPlugin($plugin);

                    $pluginBlock .= sprintf($pluginRow, $pluginUrl, $plugin->getName());
                }
            }
        }

        $sitesBlock = '';
        $sites = $this->siteService->getSitesByProfile($profileID);
        if (null === $sites || empty($sites)) {
            $sitesBlock = '<li>No sites</li>';
        } else {
            $siteRow = '<li><a href="%s">%s</a></li>';
            foreach ($sites as $site) {
                $siteUrl = $this->siteService->liveUrlForSite($site);
                $sitesBlock .= sprintf($siteRow, $siteUrl, $site->getName());
            }
        }

        $platform = $this->platformProfileService->getPlatformForProfile($profile);

        $template =<<<EOHTML
<style>
</style>
<div class="site-detail">
    <h1>Profile: {$profile->getName()}</h1>
    <div class="left graybox" style="width: 48%;">
        <table>
            <tr><th><a href="$editUrl">Edit</a></th></tr>
            <tr><th>Platform</th><td>{$platform->getName()}</td></tr>
            <tr><th>Theme</th><td>$themeName</td></tr>
            <tr><th>Plugins</th><td><ul>$pluginBlock</ul></td></tr>
            <tr><th>Sites</th><td><ul>$sitesBlock</ul></td></tr>
        </table>
    </div>
</div>
EOHTML;
        echo $template;
    }

    /**
     * Create a new profile
     */
    public function create ()
    {
        // is this access check necessary?
        if ( !current_user_can( 'manage_options' ) )  {
            wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
        }

        $profile = $this->profileService->newModel();
        if (isset($_POST['save']) && "Save Profile" === $_POST['save']) {
            $profile->setName( sanitize_text_field($_POST['name']) );
            $profile->setDescription( sanitize_text_field($_POST['description']) );
            $profile->setThemeID( sanitize_text_field($_POST['theme']) );
            //$profile->setSource( sanitize_text_field($_POST['source']) );
            $profile->setProvision( isset($_POST['provision']) && '1' == $_POST['provision'] );
            $profile->setClone( isset($_POST['clone']) && '1' == $_POST['clone'] );
            $this->profileService->createProfile($profile);
            if (isset($_POST['plugins'])) {
                foreach ($_POST['plugins'] as $id) {
                    $plugin = $this->pluginService->getPlugin($id);
                    $this->profilePluginService->addPluginToProfile($plugin, $profile);
                }
            }
            if (isset($_POST['platform'])) {
                $platform = $this->platformService->platform(sanitize_text_field($_POST['platform']));
                $this->platformProfileService->addProfileToPlatform($profile, $platform);
            }

            Helper::adminMessage( __( 'Profile created'), Helper::SUCCESS);
            $profileUrl = admin_url( 'admin.php?page=divimanager-profile-view&profile_id=' . $profile->getId() );
            echo <<<EOHTML
<p>Click to return to <a href="$profileUrl">profile</a>.</p>
EOHTML;
            return;
        }

        $theme = $profile->getThemeID() === null ? null : $profile->getThemeID();
        $themeSelect = $this->makeThemeSelect($theme);
        $platform = $this->platformProfileService->getPlatformForProfile($profile);
        $platformSelect = $this->makePlatformSelect($platform);
        //$sourceSelect = $this->makeSiteSelect($platform, $profile->getSource());
        $plugins = $this->profilePluginService->getPluginsForProfile($profile);
        $pluginCheckboxes = $this->makePluginCheckboxes($plugins);
?>
    <div class="wrap">
        <h1 class="wp-heading-inline"><?php echo esc_html( get_admin_page_title() ); ?></h1>
        <div id="post-body" class="metabox-holder columns-2">
            <div id="post-body-content">
                <div class="meta-box-sortables ui-sortable">
                    <form method="POST">
                    <table class='form-table'>
                    <tr><th>Name</th><td><input type="text" name="name" value="<?php echo $profile->getName(); ?>" /></td><td>Name of the profile.</td></tr>
                    <tr><th>Description</th><td><textarea name="description"><?php echo $profile->getDescription(); ?></textarea></td><td>A description of this profile.</td></tr>
                    <tr><th>Platform</th><td><?php echo $platformSelect; ?></td><td>Select the platform which hosts sites from this profile.</td></tr>
                    <tr><th>Theme</th><td><?php echo $themeSelect; ?></td><td>Select the theme which this profile prefers.</td></tr>
                    <tr><th>Plugins</th><td><?php echo $pluginCheckboxes; ?></td><td>Check plugins which should be included in this profile.</td></tr>
                    <!--<tr><th>Source</th><td><?php //echo $sourceSelect; ?></td><td>Select the source site for cloning.</td></tr>-->
                    </table>
                    <button type="submit" name="save" value="Save Profile">Save Profile</button>
                    <button type="reset" name="reset" value="Reset">Reset</button>
                    </form>
                </div>
            </div>
        </div>
        <br class="clear">
	</div>
<?php
    }

    /**
     * Edit a profile
     */
    public function update ()
    {
        // is this access check necessary?
        if ( !current_user_can( 'manage_options' ) )  {
            wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
        }

        $profile = $this->profileService->getProfile($_GET['profile_id']);
        if (isset($_POST['save']) && "Save Profile" === $_POST['save']) {
            $profile->setName( sanitize_text_field($_POST['name']) );
            $profile->setDescription( sanitize_text_field($_POST['description']) );
            $profile->setThemeID( sanitize_text_field($_POST['theme']) );
            //$profile->setSource( sanitize_text_field($_POST['source']) );
            $profile->setClone( isset($_POST['clone']) && '1' == $_POST['clone'] );
            $this->profileService->updateProfile($profile);
            if (isset($_POST['plugins'])) {
                foreach ($_POST['plugins'] as $id) {
                    $plugin = $this->pluginService->getPlugin($id);
                    $this->profilePluginService->addPluginToProfile($plugin, $profile);
                }
            }
            if (isset($_POST['platform'])) {
                $platform = $this->platformService->platform(sanitize_text_field($_POST['platform']));
                $oldPlatform = $this->platformProfileService->getPlatformForProfile($profile);
                if ($oldPlatform != $platform) {
                    if (null !== $oldPlatform) {
                        $this->platformProfileService->removeProfileFromPlatform($profile, $oldPlatform);
                    }
                    $this->platformProfileService->addProfileToPlatform($profile, $platform);
                }
            }
            Helper::adminMessage( __( 'Profile updated'), Helper::SUCCESS);
            $profileUrl = admin_url( 'admin.php?page=divimanager-profile-view&profile_id=' . $profile->getId() );
            echo <<<EOHTML
<p>Click to return to <a href="$profileUrl">profile</a>.</p>
EOHTML;
            return;
        }

        $theme = $profile->getThemeID() === null ? null : $profile->getThemeID();
        $themeSelect = $this->makeThemeSelect($theme);
        $plugins = $this->profilePluginService->getPluginsForProfile($profile);
        $pluginCheckboxes = $this->makePluginCheckboxes($plugins);
        $platform = $this->platformProfileService->getPlatformForProfile($profile);
        $platformSelect = $this->makePlatformSelect($platform);
        //$sourceSelect = $this->makeSiteSelect($platform, $profile->getSource());
?>
    <div class="wrap">
        <h1 class="wp-heading-inline"><?php echo esc_html( get_admin_page_title() ); ?></h1>
        <div id="post-body" class="metabox-holder columns-2">
            <div id="post-body-content">
                <div class="meta-box-sortables ui-sortable">
                    <form method="POST">
                    <table class='form-table'>
                    <tr><th>Name</th><td><input type="text" name="name" value="<?php echo $profile->getName(); ?>" /></td><td>Name of the profile, for display purposes.</td></tr>
                    <tr><th>Description</th><td><textarea name="description"><?php echo $profile->getDescription(); ?></textarea></td><td>A description of this profile.</td></tr>
                    <tr><th>Platform</th><td><?php echo $platformSelect; ?></td><td>Select the platform which hosts sites from this profile.</td></tr>
                    <tr><th>Theme</th><td><?php echo $themeSelect; ?></td><td>Select the theme which this profile prefers.</td></tr>
                    <tr><th>Plugins</th><td><?php echo $pluginCheckboxes; ?></td><td>Check plugins which should be included in this profile.</td></tr>
                    <!--<tr><th>Source</th><td><?php //echo $sourceSelect; ?></td><td>Select the source site for cloning.</td></tr>-->
                    </table>
                    <button type="submit" name="save" value="Save Profile">Save Profile</button>
                    <button type="reset" name="reset" value="Reset">Reset</button>
                    </form>
                </div>
            </div>
        </div>
        <br class="clear">
	</div>
<?php
    }

    public function makeThemeSelect($themeID = null)
    {
        $html =<<<EOHTML
<select name="theme" id="theme">
    <option value=""></option>
EOHTML;

        $option =<<<EOHTML
    <option value="%s"%s>%s</option>
EOHTML;
        foreach ($this->themeService->getThemes() as $theme) {
            $selected = '';
            if (null !== $themeID && $themeID == $theme->getId()) {
                $selected = ' selected';
            }
            $html .= sprintf($option, $theme->getId(), $selected, $theme->getName());
        }

        $html .=<<<EOHTML
</select>
EOHTML;
        return $html;
    }

    public function makePluginCheckboxes($plugins)
    {
        $html =<<<EOHTML
<div>
EOHTML;

        $checkbox =<<<EOHTML
    <input type="checkbox" name="plugins[]" value="%s"%s>%s</option><br />
EOHTML;
        foreach ($this->pluginService->getPlugins() as $plugin) {
            $selected = '';
            if (null !== $plugins) {
		foreach ($plugins as $p) {
			if ($p->getId() == $plugin->getId()) {
				$selected = ' checked=true ';
				break;
			}
		}
            }
            $html .= sprintf($checkbox, $plugin->getId(), $selected, $plugin->getName());
        }

        $html .=<<<EOHTML
</div>
EOHTML;
        return $html;
    }

    public function makePlatformSelect($platform = null)
    {
        $platformID = $platform->getId();
        $platforms = $this->platformService->activePlatforms();
        if (0 === count($platforms)) {
            return 'There are no active platforms.  Please install and configure.';
        }

        $html =<<<EOHTML
<select name="platform" id="platform">
    <option value=""></option>
EOHTML;

        $option =<<<EOHTML
    <option value="%s"%s>%s</option>
EOHTML;
        foreach ($platforms as $platform) {
            $selected = '';
            if (null !== $platformID && $platformID == $platform->getId()) {
                $selected = ' selected';
            }
            $html .= sprintf($option, $platform->getId(), $selected, $platform->getName());
        }

        $html .=<<<EOHTML
</select>
EOHTML;
        return $html;
    }

    public function makeSiteSelect($platformID, $siteID = null)
    {
        $platform = $this->platformService->platform($platformID);
        if (null === $platform) {
            return 'FIX THIS: a platform must be selected first.';
        }
        $sites = $this->platformService->getSitesOnPlatform($platform);
        if (0 === count($sites)) {
            return 'There are no sites on this platform.';
        }

        $html =<<<EOHTML
<select name="source" id="source">
    <option value=""></option>
EOHTML;

        $option =<<<EOHTML
    <option value="%s"%s>%s</option>
EOHTML;
        foreach ($sites as $site) {
            $selected = '';
            if (null !== $siteID && $siteID == $site->getId()) {
                $selected = ' selected';
            }
            $html .= sprintf($option, $site->getId(), $selected, $site->getName());
        }

        $html .=<<<EOHTML
</select>
EOHTML;
        return $html;
    }
}
