<?php declare(strict_types=1);
namespace DiviManagerPro\Platform;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Service\Site as SiteService;
use DiviManagerPro\Service\Platform as PlatformService;
use DiviManagerPro\Service\PlatformServer as PlatformServerService;

/**
 * Base class for hosting platforms which Manager supports.
 * Specifics should be provided by extension plugins
 */
abstract class AbstractPlatform implements PlatformInterface
{
    const PLATFORM_OPTION_KEY = 'platforms';

    /**
     * @var String our id for the manager
     */
    protected $id;

    /**
     * @var String A name for this profile
     */
    protected $name;

    /**
     * @var String Some description of the profile
     */
    protected $description;

    /**
     * @var String What capabilities this platform has
     */
    protected $capabilities;

    /**
     * @var bool whether the platform supports provisioning of a new site
     */
    protected $provision;

    /**
     * @var bool whether the platform supports cloning of sites
     */
    protected $clone;

    /**
     * @var String The transient field name
     */
    protected $transientField;

    /**
     * @var array Cached installs with processed data
     */
    protected $cachedInstalls;

    /**
     * @var array Cached servers with processed data
     */
    protected $cachedServers;

    /**
     * @var array Cached domains for each platform
     */
    protected $cachedDomains;

    /**
     * @var Option options manager
     */
    protected $option;

    /**
     * @var array Platform options
     */
    protected $options;

    /**
     * @var string Key in the options table
     */
    protected $optionsKey;

    public function __construct()
    {
        $this->clone = false;
        $this->transientField = 'divimanager_platforms';
        $this->options = [];
    }

    public function setSiteService(Siteservice $sm)
    {
        $this->siteService = $sm;
    }

    public function setPlatformService(PlatformService $service)
    {
        $this->platformService = $service;
    }

    public function setPlatformServerService(PlatformServerService $service)
    {
        $this->platformServerService = $service;
    }

    public function setOption($option)
    {   
        $this->option = $option;
    }

    public function loadOptions()
    {
        $options = $this->option->get('platforms');
        if (isset($options['platforms'][$this->getId()])) {
            $this->setOptions($options['platforms'][$this->getId()]);
        }
    }

    public function saveOptions()
    {
        /*
        $options = get_option($this->optionsKey);
        $options['platforms'][$this->getId()] = $this->getOptions();

        $rc = wp_cache_delete($this->optionsKey, 'options');
        $rc = update_option($this->optionsKey, $options, false);
         */
    }

    public function loadCachedData($refresh = false)
    {
        if (isset($this->cachedInstalls) && false === $refresh) {
            return true;
        }

        $this->cachedInstalls = [];
        $data = get_transient($this->transientField);

        // if false, data expired so set to an empty array
        if (false === $data) {
            $data = [];
        }

        $id = $this->getId();

        // if data is missing for this platform, recreate
        if (! isset($data[$id]) || true === $refresh) {
            $installs = $this->installs();
            $data[$id] = [];
            if (0 !== count($installs)) {
                foreach ($installs as $key => $install) {
                    //$site = $this->siteService->getSiteByPlatformInstallID($install['id']);
                    $site = $this->platformService->getSiteOnPlatformByInstallID($this, $install['id']);

                    if (null === $site) {
                        // installs can have multiple domains, scan all of them for a match
                        foreach ($install['domains'] as $domain) {
                            $site = $this->siteService->getSiteByDomain($domain);
                            if (null === $site) {
                                $site_id = '';
                            } else {
                                $site_id = 'Found by domain:' . $site->getID();
                                break;
                            }
                        }
                    } else {
                        $site_id = $site->getID();
                    }

                    $installs[$key]['site_id'] = $site_id;
                }

                $data[$id] = $installs;
            }
            $domains = $this->domains();
            $data['domains'][$id] = $domains;

            set_transient($this->transientField, $data, DAY_IN_SECONDS); // cache for up to one day
        }

        $this->cachedInstalls = $data[$id];
        $this->cachedDomains = $data['domains'][$id];
        $this->cachedServers = $this->servers();

        return true;
    }

    public function getInstalls($refresh = false)
    {
        $this->loadCachedData($refresh);
        return $this->cachedInstalls;
    }

    public function getServers($refresh = false)
    {
        $this->loadCachedData($refresh);
        return $this->cachedServers;
    }

    public function getDomains($refresh = false)
    {
        $this->loadCachedData($refresh);
        return $this->cachedDomains;
    }

    public function checkDomain($domain)
    {
        //$response = \dns_get_record($domain);
        $response = \checkdnsrr($domain, 'A');
        return $response;
    }

    public function filesizes($site)
    {
        return [];
    }

    public function toArray()
    {
        return [
            'id' => $this->getId(),
            'name' => $this->getName(),
            'description' => $this->getDescription(),
            'capabilities' => $this->getCapabilities(),
            'provision' => $this->getProvision(),
            'clone' => $this->getClone(),
        ];
    }

    public function fromArray(array $data)
    {
        $this->setId($data['id']);
        $this->setName($data['name']);
        $this->setDescription($data['description']);
        $this->setCapabilities($data['capabilities']);
        $this->setProvision($data['provision']);
        $this->setClone($data['clone']);

        return $this;
    }

    public function getId()
    {
        return $this->id;
    }
    public function setId($newId)
    {
        $this->id = $newId;
        return $this;
    }

    public function getName()
    {
        return $this->name;
    }
    public function setName($newName)
    {
        $this->name = $newName;
        return $this;
    }

    public function getDescription()
    {
        return $this->description;
    }
    public function setDescription($newDescription)
    {
        $this->description = $newDescription;
        return $this;
    }

    public function getCapabilities()
    {
        return $this->capabilities;
    }
    public function setCapabilities($newCapabilities)
    {
        $this->capabilities = $newCapabilities;
        return $this;
    }

    public function getProvision()
    {
        return $this->provision;
    }
    public function setProvision(bool $newProvision)
    {
        $this->provision = $newProvision;
        return $this;
    }

    public function getClone()
    {
        return $this->clone;
    }
    public function setClone(bool $newClone)
    {
        $this->clone = $newClone;
        return $this;
    }

    public function getPlugins()
    {
        return $this->plugins;
    }
    public function setPlugins(array $newPlugins = null)
    {
        if (null === $newPlugins) {
            $newPlugins = [];
        }
        $this->plugins = $newPlugins;
        return $this;
    }

    public function getOptions()
    {
        if (empty($this->options)) {
            $options = $this->option->getOption(self::PLATFORM_OPTION_KEY);
            if (isset($options[$this->getId()])) {
                $this->options = $options[$this->getId()];
            }
        }
        return $this->options;
    }

    public function setOptions(array $newOptions = null)
    {
        if (null === $newOptions) {
            $newOptions = [];
        }
        $this->options = $newOptions;
        return $this;
    }

    /**
     * Helpers to check if this profile uses clone or does a new install
     *
     * @return bool
     */
    public function doesClone()
    {
        return true === $this->clone;
    }
    public function doesProvision()
    {
        return true === $this->provision;
    }

    /**
     * API Wrapper
     *
     * These are the behaviors expected from all platforms
     */

    /**
     * List the sites hosted on the platform
     */
    abstract public function installs();

    /**
     * Get platform details for a site
     */
    abstract public function getSite($site);

    /**
     * Provision a new site on the platform
     */
    abstract public function provision($name);

    /**
     * Clone an existing site on the platform
     */
    abstract public function cloneInstall($sourceId, $name);

    /**
     * Deactivate a site
     */
    abstract public function deactivate($id);

    /**
     * Change setting of a site
     */
    abstract public function alter($id, $setting, $value);
}
