<?php declare(strict_types=1);
namespace DiviManagerPro\Serverpilot\Platform;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use ServerPilotAPI\ServerPilotAPI;
use DNSMadeEasyAPI\DNSMadeEasyAPI;
//use DigitalOceanAPI\DigitalOceanAPI;
use DiviManager\Model\Site as SiteModel;
use DiviManagerPro\Serverpilot\Action\Site as SiteAction;
use DiviManagerPro\Platform\AbstractPlatform;

/**
 * Implement the platform api for the dme/do/sp platform combination
 */
class Serverpilot extends AbstractPlatform
{
    public $host;
    public $clientID;
    public $apiKey;
    public $server;
    public $sshKey;
    public $dme;

    public function __construct()
    {
        $this->setId('serverpilot');
        $this->setName('Serverpilot');
        $this->setDescription('Work with the ServerPilot server and website management tool');
        $this->setCapabilities('Create new sites');
        $this->setProvision(true);
        $this->setClone(false);

        parent::__construct();
    }

    public function getSPAPI()
    {
        $options = $this->getOptions();
        $spapi = new ServerPilotAPI($options['api_url'], $options['client'], $options['api_key']);
        return $spapi;
    }

    public function getDMEAPI()
    {
        $options = $this->getOptions();
        $dmeapi = new DNSMadeEasyAPI($options['dmeapiurl'], $options['dmeapikey'], $options['dmesecretkey']);
        return $dmeapi;
    }

    /**
     * List the sites hosted on the platform
     */
    public function installs()
    {
        $spapi = $this->getSPAPI();
        $sites = $spapi->sites();
        $spapi->close();

        return $sites;
    }

    /**
     * List the servers registered with the platform
     */
    public function servers()
    {
        $spapi = $this->getSPAPI();
        $sites = $spapi->servers();
        $spapi->close();

        return $sites;
    }

    /**
     * Get the registered domains on dme
     */
    public function domains()
    {
        $options = $this->getOptions();
        $dmeapi = $this->getDMEAPI();
        return $dmeapi->getRecords($options['dmedomainid']);
    }

    /**
     * Get filesizes for site
     * return wp-content and db usage, maybe more
     */
    public function filesizes($site)
    {
        $paths = $this->paths($site->getInstallID());
        var_dump($paths);
        //$filesize = `du -s {$paths[]}`;
        return [];
    }

    /**
     * Get platform details for a site
     */
    public function getSite($site)
    {
        $installID = $site->getInstallID();
        if (null === $installID) {
            return [];
        }
        $spapi = $this->getSPAPI();
        $site = $spapi->site($installID);
        $spapi->close();

        return $site;
    }

    /**
     * Provision a new site on the platform
     *
     * $name is the sld, the shepherdsloft part of shepherdsloft.com
     * $name is used for the subdomain, the site name, and the site title
     */
    public function provision($name)
    {
        error_log(
            __CLASS__.__FUNCTION__.
            ': Beginning install for site: ' . $name
        );

        // add to dme
        $this->registerDomain($name);

        $options = $this->getOptions();
        $spapi = $this->getSPAPI();
        $spapi->setAdminUser($options['adminuser']);
        $spapi->setAdminEmail($options['adminemail']);

        $length = strlen($name);
        if (32 < $length) {
            $length = 32;
        }
        $truncatedUserName = substr($name, 0, $length);
        $user = $spapi->createUser($options['server'], $truncatedUserName, bin2hex(\random_bytes(10)), $options['sshkey']);
        $actionid = $user->actionid;
        while(true === $spapi->actionOpen($actionid)) {
            error_log(
                __CLASS__.__FUNCTION__.
                ': Waiting on createUser action to complete'
            );
            $this->doNothing();
        }
        if (false === $spapi->actionSuccess($actionid)) {
            throw new \RuntimeException('Error creating user for new install');
        }
        $domain = $name . '.' . $options['dmestaging'];
        $site = $spapi->createSite($name, $user->data->id, $domain, $options['runtime']);
        $actionid = $site->actionid;
        while(true === $spapi->actionOpen($actionid)) {
            var_dump('waiting on createSite action to complete');
            $this->doNothing();
        }
        if (false === $spapi->actionSuccess($actionid)) {
            throw new \RuntimeException('Error creating site for new install');
        }

        error_log(
            __CLASS__.__FUNCTION__.
            ': Finished install for site: ' . $name
        );

        return $site->data->id;
    }

    /**
     * Migrate one site to another
     */
    public function migrateSite($sourceID, $siteID)
    {
        $spapi = $this->getSPAPI();

        $src = $spapi->paths($sourceID);
        $dest = $spapi->paths($siteID);

        list($fromUser, $fromApp) = $src;
        list($toUser, $toApp) = $dest;
        $siteAction = new SiteAction;
        return $siteAction->migrate($fromUser, $fromApp, $toUser, $toApp);
    }

    /**
     * Clone an existing site on the platform
     */
    public function cloneInstall($sourceID, $name)
    {
        $spapi = $this->getSPAPI();

        try {
            // create the new site, which also adds to DME
            $siteID = $this->newInstall($name);
            // migrate source to new site
            //$this->migrateSite($sourceID, $siteID);
        } catch (\Exception $e) {
            throw new \RuntimeException('Failed to clone to new site: '. $e->getMessage());
        }

        return $siteID;
    }

    /**
     * Get path information for site
     */
    public function paths ($siteID)
    {
        $spapi = $this->getSPAPI();
        return $spapi->paths($siteID);
    }

    /**
     * Deactivate a site
     */
    public function deactivate($id)
    {
    }

    /**
     * Change setting of a site
     */
    public function alter($id, $setting, $value)
    {
    }

    /**
     * Get required options for the Manager to build the form
     */
    public function options()
    {
        throw new \Exception('DEPRECATED');
        /*
        return [
            ['name' => 'License Key', 'type' => 'text', 'field' => 'license', 'description' => 'Your license key for this plugin'],
            ['name' => 'ServerPilot API Url', 'type' => 'text', 'field' => 'spApiUrl', 'description' => 'ServerPilot API URL, should be set to https://api.serverpilot.io'],
            ['name' => 'ServerPilot Client ID', 'type' => 'text', 'field' => 'spClientID', 'description' => 'Client ID for the ServerPilot account'],
            ['name' => 'ServerPilot API Key', 'type' => 'password', 'field' => 'spApiKey', 'description' => 'API Key for the client ID corredsponding to the ServerPilot account'],
            ['name' => 'ServerPilot Server ID', 'type' => 'text', 'field' => 'spServer', 'description' => 'ID corresponding to the server on Serverpilot to which new sites will be installed'],
            ['name' => 'ServerPilot SSH Key', 'type' => 'text', 'field' => 'spSshKey', 'description' => 'SSH Key to associate with the new site, if you have one'],
            ['name' => 'ServerPilot Default Admin User', 'type' => 'text', 'field' => 'spAdminUser', 'description' => 'Default admin user for new sites'],
            ['name' => 'ServerPilot Default Admin Email', 'type' => 'text', 'field' => 'spAdminEmail', 'description' => 'Default admin email for new sites'],
            ['name' => 'ServerPilot Default PHP Runtime', 'type' => 'text', 'field' => 'spPhpRuntime', 'description' => 'Default php runtime for new sites'],
            ['name' => 'DME API Url', 'type' => 'text', 'field' => 'dmeApiUrl', 'description' => 'Dnsmadeeasy API URL, should be set to https://api.dnsmadeeasy.com/'],
            ['name' => 'DME API Key', 'type' => 'password', 'field' => 'dmeApiKey', 'description' => 'API key for dnsmadeeasy service'],
            ['name' => 'DME Secret Key', 'type' => 'password', 'field' => 'dmeSecretKey', 'description' => 'Secret key for dnsmadeeasy service'],
            ['name' => 'DME Domain ID', 'type' => 'text', 'field' => 'dmeDomainID', 'description' => 'Domain ID on dnemadeeasy to which the staging domains will be added'],
            ['name' => 'DME Server IP', 'type' => 'text', 'field' => 'dmeServerIP', 'description' => 'IP of the server on which the sites are being installed'],
            ['name' => 'Staging Domain', 'type' => 'text', 'field' => 'stagingDomain', 'description' => 'The domain the websites will be added to as subdomains'],
            //['name' => '', 'type' => 'text', 'field' => '', 'description' => ''],

        ];
         */
    }

    public function doNothing()
    {
        sleep(1);
    }

    /**
     * Enable auto-ssl on site
     */
    public function enableAutoSsl($site)
    {
        $spapi = $this->getSPAPI();
        $response = $spapi->autosslSite($site->getInstallID());
        $actionid = $response->actionid;
        while(true === $spapi->actionOpen($actionid)) {
            //var_dump('waiting on autossl action to complete');
            $this->doNothing();
        }
        if (false === $spapi->actionSuccess($actionid)) {
            throw new \RuntimeException('Error doing autossl for site');
        }
        $spapi->close();
    }

    /**
     * Enable auto-ssl and force-ssl on site
     */
    public function enableForceSsl($site)
    {
        $spapi = $this->getSPAPI();
        $response = $spapi->forcesslSite($site->getInstallID());
        $actionid = $response->actionid;
        while(true === $spapi->actionOpen($actionid)) {
            //var_dump('waiting on autossl action to complete');
            $this->doNothing();
        }
        if (false === $spapi->actionSuccess($actionid)) {
            throw new \RuntimeException('Error doing autossl for site');
        }
        $spapi->close();
    }

    /**
     * Add a domain to the site
     */
    public function addDomainToSite($domain, $site)
    {
        $spapi = $this->getSPAPI();
        $response = $spapi->addDomainToSite($site->getInstallID(), $domain);
        $actionid = $response->actionid;
        while(true === $spapi->actionOpen($actionid)) {
            var_dump('waiting on addDomain action to complete');
            $this->doNothing();
        }
        if (false === $spapi->actionSuccess($actionid)) {
            throw new \RuntimeException('Error doing addDomain for site');
        }
        $spapi->close();
    }

    /**
     * Add domain to dnsmadeeasy
     */
    public function registerDomain($domain)
    {
        $options = $this->getOptions();

        // add to dme
        $dmeapi = $this->getDMEAPI();
        $dmeapi->addRecord($domain, $options['dmedomainid'], $options['dmeserverip']);
    }
}
