<?php declare(strict_types=1);
namespace DiviManagerPro\Serverpilot;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Registry;
use DiviManager\Helper;
use DiviManager\Helper\Option;
use DiviManagerPro\Service\Platform as PlatformService;
use DiviManagerPro\Serverpilot\Admin;

class Serverpilot
{
    /**
     * var string
     *
     * The hook prefix for our actions so we can hook into events and stay synchronized with the main plugin
     */
    protected $hookPrefix;

    public function __construct($hookPrefix)
    {
        $this->hookPrefix = $hookPrefix;

        add_action( 'plugins_loaded', [$this, 'plugins_loaded'] );
        add_action( 'init', [$this, 'wp_init'], 12);
        add_action( $this->hookPrefix . '_admin_init', [$this, 'admin_init'], 12);
        add_action( $this->hookPrefix . '_activation', [$this, 'activation'], 12);
        add_action( $this->hookPrefix . '_deactivation', [$this, 'deactivation'], 12);
        add_action( $this->hookPrefix . '_uninstall', [static::class, 'uninstall'], 12);
        add_action( $this->hookPrefix . '_services_init', [$this, 'services_init'], 12);
    }

    /**
     * function for wordpress init hook
     */
    public function wp_init()
    {
        $this->update_db_check();

        // add wp-cli command
        if ( defined( 'WP_CLI' ) && WP_CLI ) {
            //if ($this->isServerpilotEnabled()) {
                //$this->registry->setService(Command\Report::class, Command\ReportFactory::class);
                //$reportCommand = $this->registry->getService(Command\Report::class);
                //\WP_CLI::add_command( 'manager report', [$reportCommand, 'report'] );
            //}
        }

        if (is_admin() || (defined('WP_CLI') && WP_CLI)) {
        }
    }

    /**
     * Runs when the admin_init action is called, admin pages are being viewed
     *
     * No need to use an is_admin() check as this is only run before loading admin pages
     */
    public function admin_init()
    {
        $this->registry->getService(Admin::class)->init();
		$this->option = $this->registry->getService(Option::class); 

        // register our platform class
        if ($this->isServerpilotEnabled()) {
            $pm = $this->registry->getService(PlatformService::class);
            $pm->register($this->registry->getService(Platform\Serverpilot::class));
        }
    }

    /**
     * Setup our services.
     *
     * Every plugin will be adding their own
     */
    public function services_init(Registry $registry)
    {
        $registry->setService(__CLASS__, $this); 
        $registry->setService(Admin::class, AdminFactory::class);
        $registry->setService(Platform\Serverpilot::class, Platform\ServerpilotFactory::class);
        $registry->setService(Action\Site::class, Action\SiteFactory::class);
    }

    /**
     * install db schema
     *
     * detects the version of the schema
     * if different, runs the script
     */
    protected function install ()
    {
        global $wpdb;

        $installed_ver = get_option( self::VERSION_NAME );
return;
        if ( $installed_ver != self::VERSION ) {

            $table_name = $wpdb->prefix . 'dmpro_platform'; 

            $charset_collate = $wpdb->get_charset_collate();

            // wpprsprv_site is the table storing entries for each provision.
            // it links to the customer, the order and specific product in the order, plus the domain entered and which tier/profile/clone target was created
            $sql =<<<EOSQL
CREATE TABLE $table_name (
id int NOT NULL AUTO_INCREMENT,
domain varchar(255) NOT NULL,
customer_id integer NOT NULL,
product_id integer NOT NULL,
order_id integer NOT NULL,
profile varchar(255) NOT NULL,
status varchar(255) DEFAULT '' NOT NULL,
time datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
last text NULL,
data text NULL,
PRIMARY KEY  (id)
) $charset_collate;
EOSQL;

            require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
            dbDelta( $sql );
            update_option( self::VERSION_NAME, self::VERSION, FALSE );
        }
    }

    /**
     * Runs when the plugin has been first deactivated and then deleted
     *
     * Removes all data related to the plugin
     */
    public static function uninstall ()
    {
            $sql =<<<EOSQL
DROP TABLE $table_name;
EOSQL;

            require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
            dbDelta( $sql );
    }

    public function update_db_check()
    {
        return;
        if ( get_site_option( self::VERSION_NAME ) != self::VERSION ) {
            $this->install();
        }
    }

    /**
     * On activation of this plugin
     * Also activate the monolog handler
     * @see https://github.com/bradmkjr/monolog-wordpress
     */
    public function activation()
    {
        $this->install();
    }

    /**
     * On deactivation of this plugin
     * Also deactive the monolog handler
     * @see https://github.com/bradmkjr/monolog-wordpress
     */
    public function deactivation()
    {
    }

    /**
     * Runs when the plugins_loaded action is called
     * Use this to do db updates
     *
     * Run actions which need to occur every page request, but before wordpress init
     */
    public function plugins_loaded()
    {
        $this->divimanager = \DiviManager\DiviManager::getInstance(); // get the main plugin's instance for use
		$this->registry = $this->divimanager->getRegistry();
    }

    /**
     * Whether the Serverpilot functionality is enabled
     *
     * @return bool
     */
    public function isServerpilotEnabled()
    {
        $this->option = $this->registry->getService(Option::class);
        return true == $this->option->getOption('serverpilot_enabled');
    }
}
