<?php declare(strict_types=1);
namespace DiviManagerPro\Service;

defined('ABSPATH') or die('Unexpected access');

use DiviManagerPro\Model\Platform as PlatformModel;
use DiviManagerPro\Platform\PlatformInterface;
use DiviManager\Model\Site as SiteModel;
use DiviManager\Service\Site as SiteService;
use DiviManager\Service\DbManager;

/**
 * Keep track of the platforms installed
 *
 * Do we need a central manager for the platforms?
 * Would multiple be installed?
 * Need to be able to identify the platform
 * Probably require at least one to be installed
 */
class Platform extends DbManager
{
    /**
     * @var array platforms registered with the manager
     */
    protected $platforms = [];

    /**
     * @var SiteService
     */
    protected $siteService;

    public function __construct()
    {
        $fields =<<<EOSQL
site_id int NOT NULL,
platform_id varchar(64) NOT NULL,
install_id varchar(64) NOT NULL
EOSQL;

        parent::__construct('dm_platforms_sites', $fields);
    }

    public function setSiteService(SiteService $s)
    {
        $this->siteService = $s;
    }

    /**
     * Return the currently active platforms
     * There extension is installed and activated
     * Platform may not be configured
     */
    public function activePlatforms()
    {
        return $this->platforms;
    }

    /**
     * Register a platform with the manager
     * These are added by extensions, not database stored
     */
    public function register($platform)
    {
        $this->platforms[$platform->getId()] = $platform;
    }

    /**
     * Get a specific registered platform
     */
    public function platform($platform_id)
    {
        $platforms = $this->activePlatforms();
        if (isset($platforms[$platform_id])) {
            return $platforms[$platform_id];
        }

        return null;
    }

    /**
     * Get site for given platform by installID
     *
     * @param PlatformInterface
     * @param installID
     * @return Site
     */
    public function getSiteOnPlatformByInstallID( PlatformInterface $platform, $installID )
    {
        $ids = $this->getFieldByField('site_id', ['platform_id' => $platform->getID(), 'install_id' => $installID]);

        if (null === $ids) {
            return null;
        }

        $id = array_pop($ids);

        if (null === $id) {
            return null;
        }

        return $this->siteService->getSite($id);
    }

    /**
     * Get sites for given platform
     *
     * @param PlatformInterface
     * @return array
     */
    public function getSitesOnPlatform( PlatformInterface $platform )
    {
        $ids = $this->getFieldByField('site_id', ['platform_id' => $platform->getID()]);

        if (null === $ids) {
            return [];
        }

        return $this->siteService->getSitesByID($ids);
    }

    /**
     * Get sites on multiple platforms
     *
     * @param Site
     * @return array
     */
    public function getSitesOnPlatforms( array $platforms, $options = null )
    {
        if (empty($platforms)) {
            return [];
        }

        $ids = [];
        foreach ($platforms as $platform) {
            $ids[] = $platform->getId();
        }

        $ids = $this->getFieldByField('site_id', ['platform_id' => $ids]);

        if (null === $ids) {
            return [];
        }

        return $this->siteService->getSitesByID($ids);
    }

    /**
     * Add site to platform
     *
     * @param PlatformInterface
     * @param SiteModel
     */
    public function addSiteToPlatform( PlatformInterface $platform, SiteModel $site, $installID)
    {
        // TODO: add check if already exists

        $this->wpdb->insert(
            $this->getTableName(),
            [
                'site_id' => $site->getId(),
                'platform_id' => $platform->getId(),
                'install_id' => $installID,
            ],
            [
                '%d',
                '%s',
                '%s',
            ]
        );
    }

    /**
     * Remove site from platform
     *
     * @param PlatformInterface
     * @param SiteModel
     */
    public function removeSiteFromPlatform( PlatformInterface $plugin, SiteModel $site)
    {
        // TODO: check if platform and site are joined, or let the delete statement run even if it deletes zero rows?
        $this->wpdb->delete(
            $this->tableName,
            [
                'site_id' => $site->getId(),
                'platform_id' => $platform->getId(),
            ],
            [
                '%d',
                '%s',
            ]
        );
    }
}
