<?php declare(strict_types=1);
namespace DiviManagerPro\Service;

use DiviManager\Service\DbManager;
use DiviManager\Model\Profile as ProfileModel;
use DiviManagerPro\Platform\PlatformInterface as PlatformModel;
use DiviManager\Service\Profile as ProfileService;
use DiviManagerPro\Service\Platform as PlatformService;

/**
 * Service class for working with platform profile joins
 */
class PlatformProfile extends DbManager
{
    public function __construct()
    {
        $fields =<<<EOSQL
platform_id varchar(64) NULL,
profile_id int NOT NULL
EOSQL;

        parent::__construct('divimanager_pro_platforms_profiles', $fields);
    }

    /**
     * Set the platform service
     */
    public function setPlatformService(PlatformService $p)
    {
        $this->platformService = $p;
    }

    /**
     * Set the profile service
     */
    public function setProfileService(ProfileService $s)
    {
        $this->profileService = $s;
    }

    /**
     * Get profiles for given platform
     *
     * @param Platform
     * @return array
     */
    public function getProfilesForPlatform( PlatformModel $platform )
    {
        $ids = $this->getFieldByField('profile_id', ['platform_id' => $platform->getID()]);

        if (null === $ids) {
            return [];
        }

        return $this->profileService->getProfilesByID($ids);
    }

    /**
     * Get profiles for multiple platforms
     *
     * @param Platform
     * @return array
     */
    public function getProfilesForPlatforms( array $platforms = [], $options = null )
    {
        if (empty($platforms)) {
            return [];
        }

        $platformIDs = [];
        foreach ($platforms as $platform) {
            $platformIDs[] = $platform->getId();
        }

        $ids = $this->getFieldByField('profile_id', ['platform_id' => $platformIDs]);

        if (null === $ids) {
            return [];
        }

        return $this->profileService->getProfilesByID($ids);
    }

    public function getPlatformForProfile($profile)
    {
        $id = $this->getFieldByField('platform_id', ['profile_id' => $profile->getId()]);

        if (null === $id) {
            return null;
        }

        if (is_array($id)) {
            $id = current($id);
        }

        return $this->platformService->platform($id);
    }

    /**
     * Add profile to platform
     *
     * @param ProfileModel
     * @param PlatformModel
     */
    public function addProfileToPlatform( ProfileModel $profile, PlatformModel $platform)
    {
        $this->wpdb->insert(
            $this->getTableName(),
            [
                'platform_id' => $platform->getId(),
                'profile_id' => $profile->getId(),
            ],
            [
                '%s',
                '%d',
            ]
        );
    }

    /**
     * Remove profile from platform
     *
     * @param ProfileModel
     * @param PlatformModel
     */
    public function removeProfileFromPlatform( ProfileModel $profile, PlatformModel $platform)
    {
        $this->wpdb->delete(
            $this->tableName,
            [
                'platform_id' => $platform->getId(),
                'profile_id' => $profile->getId(),
            ],
            [
                '%s',
                '%d',
            ]
        );
    }
}
