<?php declare(strict_types=1);
namespace DiviManagerPro\Table;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Service\Site as SiteService;
use DiviManager\Service\Queue as QueueService;
use DiviManager\Service\Profile as ProfileService;
use DiviManager\Service\Theme as ThemeService;
use DiviManager\Service\Plugin as PluginService;
use DiviManager\Service\Worker as WorkerService;
use DiviManager\Action\Site as SiteAction;
use DiviManager\Model\Site as SiteModel;
use DiviManager\Helper;
use DiviManagerPro\Service\Platform as PlatformService;

/**
 * Extend WP_List_Table
 *
 * @see https://www.sitepoint.com/using-wp_list_table-to-create-wordpress-admin-tables/
 */
if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

/**
 * List all the installs(sites) on a platform
 */
class PlatformInstall extends \WP_List_Table
{
    public $stats;

    /**
     * @var PlatformService
     */
    protected $platformService;

    /**
     * @var SiteService
     */
    protected $siteService;

	public function __construct()
	{
		parent::__construct([
			'singular' => __( 'Install' ),
			'plural'   => __( 'installs' ),
            'ajax'     => false,
            'screen'   => 'platforminstalltable',
        ]);

        $this->stats = [
            'total' => 0,
            'missing' => 0,
            'found' => 0,
            'matched' => 0,
        ];

	}

    public function setPlatformService(PlatformService $pm)
    {
        $this->platformService = $pm;
    }

    public function setSiteService(SiteService $s)
    {
        $this->siteService = $s;
    }

    public function no_items()
    {
        _e( 'No installs on platform' );
    }

    /**
     * Columns to display
     *
     * @return array
     */
    public function get_columns()
    {
        $columns = [
            'cb'      => '<input type="checkbox" />',
            'name'    => __( 'Name' ),
            'install' => __( 'Install ID' ),
            'server'  => __( 'Server ID' ),
            'site'    => __( 'Site' ),
        ];

        return $columns;
    }

	/**
	 * Sortable columns
	 *
	 * @return array
	 */
    public function get_sortable_columns()
    {
		$columns = [
			'name' => [ 'name', false ],
        ];

		return $columns;
    }

    /**
     * Render the bulk action checkbox
     *
     * @param array $install
     *
     * @return string
     */
    function column_cb( $install ) {
        return sprintf(
            '<input type="checkbox" name="bulk-action[]" value="%s" />', $install['id']
        );
    }
    /**
     * Default render of column value
     *
     * @param array $install
     * @param string $column
     *
     * @return mixed
     */
    public function column_default( $install, $column )
    {
        switch ( $column ) {
        case 'name':
            return sprintf('<span class="">%s</span>', $install['name']);

        case 'install':
            return sprintf('<span class="">%s</span>', $install['id']);

        case 'server':
            return sprintf('<span class="">%s</span>', $install['serverid']);

        case 'site':
            if (empty($install['site_id'])) {
                $this->stats['missing'] += 1;
            } elseif (is_integer($install['site_id'])) {
                $this->stats['matched'] += 1;
            } else {
                $this->stats['found'] += 1;
            }
            return sprintf('<span class="">%s</span>', $install['site_id']);

        default:
            // an unmatched column, show whole row
            return print_r( $install, true );
        }
    }

	/**
	 * Define the bulk actions this table supports
	 *
	 * @return array
	 */
	public function get_bulk_actions() {
	    $actions = [
		    //'bulk-delete'   => 'Delete',
		    'bulk-import'   => 'Import Installs',
		    //'bulk-theme'    => 'Update Theme',
		    //'bulk-refresh'  => 'Refresh',
	    ];

	    return $actions;
	}

    /**
     * Prepare the data for the table
     */
    public function prepare_items()
    {
        $this->_column_headers = $this->get_column_info();
        $this->process_bulk_action();

        $perPageOption = get_current_screen()->get_option('per_page');
        //$perPage    = $this->get_items_per_page( 'installs_per_page' , $perPageOption['default'] );
        $pageNumber = $this->get_pagenum();

        $platform_id = $_GET['platform'];
        $platform = $this->platformService->platform($platform_id);
        $installs = $platform->getInstalls();

        $this->set_pagination_args( [
            'total_items' => count($installs),
            //'per_page'    => $perPage
        ] );

        $options = ['asArray' => false];
        if (isset($_GET['orderby'])) {
            $options['orderby'] = $_GET['orderby'];
        }
        if (isset($_GET['order'])) {
            $options['order'] = $_GET['order'];
        }
        if (isset($_REQUEST['s'])) {
            $options['s'] = $_REQUEST['s'];
        }
        if (isset($_REQUEST['profile'])) {
            $options['profile'] = $_REQUEST['profile'];
        }

        $this->items = $installs;
        $this->stats['total'] = count($installs);
    }

    public function check_nonce()
    {
        $nonce = esc_attr( $_REQUEST['_wpnonce'] );
        $result =  wp_verify_nonce( $nonce, 'sl_manager_bulk_action' );
        if ( ! $result) {
            wp_die('error with nonce');
        }

        if ( !current_user_can( 'manage_options' ) )  {
            wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
        }
    }
    /**
     * Process bulk and single action links
     */
    public function process_bulk_action() {
        if (empty($_POST) && ! empty($this->current_action())) {
            $this->check_nonce();
            Helper::adminMessage('Attempted to process bulk action without correct option', Helper::ERROR);
        } else {
            // forms have their own nonce processing
            if ( ( isset( $_POST['action'] ) && $_POST['action'] == 'bulk-import' )
                || ( isset( $_POST['action2'] ) && $_POST['action2'] == 'bulk-import' )
                ) {

                $ids = esc_sql( $_POST['bulk-action'] );

                $platform_id = $_GET['platform'];
                $platform = $this->platformService->platform($platform_id);
                $installs = $platform->getInstalls();
                foreach ($installs as $key => $install) {
                    if (! in_array($install['id'], $ids)) {
                        continue;
                    }
                    if (isset($install['domain'])) {
                        $domains = [$this->siteService->justDomain($install['domain'])];
                    } elseif (isset($install['domains'])) {
                        $domains = $install['domains'];
                    }
                    $found = false;
                    foreach ($domains as $domain) {
                        $site = $this->siteService->getSiteByDomain($domain);
                        if ($site) {
                            // site already exists
                            // maybe update the order and customer?
                            $site->setTimestamp(null);
                            $this->siteService->updateSite($site);
                            $this->platformService->addSiteToPlatform($platform, $site, $install['id']);
                            $found = true;
                            break;
                        }
                    }
                    if (false === $found) {
                        $domain = $domains[0]; // use the first domain in the list
                        $site = $this->siteService->newModel();
                        $site->setUrl( $domain );
                        $site->setDomain( $domain );
                        $site->setName( $domain );
                        $site->setTimestamp(null);
                        $site->setUserID( 0 );
                        $this->siteService->createSite($site);
                        $this->platformService->addSiteToPlatform($platform, $site, $install['id']);
                    }
                }

                $platform->getInstalls(true);
            }
        }
    }

    public function getStats()
    {
        return $this->stats;
    }
}
