<?php declare(strict_types=1);
namespace DiviManagerPro\Table;

defined( 'ABSPATH' ) or die( 'Unexpected access' );

use DiviManager\Table\AbstractTable;
use DiviManager\Service\Site as SiteService;
use DiviManager\Service\Queue as QueueService;
use DiviManager\Service\Profile as ProfileService;
use DiviManager\Service\Theme as ThemeService;
use DiviManager\Service\Plugin as PluginService;
use DiviManager\Service\Worker as WorkerService;
use DiviManager\Action\Site as SiteAction;
use DiviManager\Model\Site as SiteModel;
use DiviManager\Helper;
use DiviManagerPro\Service\Platform as PlatformService;
use DiviManagerPro\Service\PlatformServer as PlatformServerService;
use DiviManagerPro\Service\Server as ServerService;

/**
 * Extend WP_List_Table
 *
 * @see https://www.sitepoint.com/using-wp_list_table-to-create-wordpress-admin-tables/
 */
if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

/**
 * List all the servers on a platform
 */
//class PlatformServer extends \WP_List_Table
class PlatformServer extends AbstractTable
{
    public const HOOK_PREFIX = 'hdm_pst';
    public $stats;
    public $bulk_nonce;

    /**
     * @var PlatformService
     */
    protected $platformService;

    /**
     * @var PlatformServerService
     */
    protected $platformServerService;

    /**
     * @var ServerService
     */
    protected $serverService;

	public function __construct()
    {
        $this->init();

		parent::__construct([
			'singular' => __( 'Install' ),
			'plural'   => __( 'installs' ),
            'ajax'     => false,
            'screen'   => 'platformservertable',
            'hookPrefix' => self::HOOK_PREFIX,
        ]);

        $this->stats = [
            'total' => 0,
            'missing' => 0,
            'found' => 0,
            'matched' => 0,
        ];

	}

    public function setPlatformService(PlatformService $pm)
    {
        $this->platformService = $pm;
    }

    public function setPlatformServerService(PlatformServerService $s)
    {
        $this->platformServerService = $s;
    }

    public function setServerService(ServerService $s)
    {
        $this->serverService = $s;
    }

    public function init()
    {
        $this->bulk_nonce = wp_create_nonce( 'dm_bulk_action' );
        \add_filter (self::HOOK_PREFIX . '_table_columns_init', [$this, 'myColumns'], 1, 1);
        \add_filter (self::HOOK_PREFIX . '_table_actions_init', [$this, 'myActions'], 1, 1);
        \add_filter (self::HOOK_PREFIX . '_table_quickactions_init', [$this, 'myQuickActions'], 1, 2);
    }

    /**
     * Initialize the columns definition
     *
	 * @return array
     */
    public function myColumns($columns)
    {
        $columns['name'] = ['label' => __( 'Name' ), 'sortable' => true];
        $columns['server'] = ['label' => __( 'Server' ), 'sortable' => false];
        $columns['ip'] = ['label' => __( 'IP' ), 'sortable' => false];
        $columns['registered'] = ['label' => __( 'Registered' ), 'sortable' => false];

        return $columns;
    }

	/**
	 * Define the bulk actions this table supports
	 *
	 * @return array
	 */
    public function myActions($actions)
    {
	    $actions['bulk-delete'] = ['label' => 'Delete', 'bulk' => true, 'callback' => [$this, 'action_bulk_delete']];
        $actions['delete'] = ['label' => 'Delete', 'bulk' => false, 'callback' => [$this, 'action_delete']];
	    $actions['bulk-import'] = ['label' => 'Import', 'bulk' => true, 'callback' => [$this, 'action_bulk_import']];
        $actions['import'] = ['label' => 'Import', 'bulk' => false, 'callback' => [$this, 'action_import']];

	    return $actions;
    }

    /**
     * Prepare the quick actions for the row
     *
	 * @return array
     */
    public function myQuickActions($actions, $server)
    {
        $server_id = $server->id;
        $platform_id = $_GET['platform'];
        $importUrl = esc_url ( add_query_arg( ['page' => 'divimanager-platform-servers', 'platform' => $platform_id, 'server_id' => $server_id, 'action' => 'import', '_wpnonce' => $this->bulk_nonce], admin_url('admin.php')));
        $deleteUrl = esc_url ( add_query_arg( ['page' => 'divimanager-platform-servers', 'platform' => $platform_id, 'server_id' => $server_id, 'action' => 'delete', '_wpnonce' => $this->bulk_nonce], admin_url('admin.php')));

        $actions['import'] = sprintf( '<a href="%s">Import</a>', $importUrl);
        $actions['delete'] = sprintf( '<a href="%s">Delete</a>', $deleteUrl);

        return $actions;
    }

    /**
     * Prepare quick actions for the server
     */
    public function quickActionsForServer($server)
    {
        $actions = [
        ];
        return \apply_filters ($this->hookPrefix . '_table_quickactions_init', $actions, $server);
    }

    /**
     * Override the default behavior
     * Render the bulk action checkbox
     *
     * @param array $server
     *
     * @return string
     */
    public function column_cb( $server )
    {
        return sprintf(
            '<input type="checkbox" name="bulk-action[]" value="%s" />', $server->id
        );
    }

    /***
     * Columns
     */
    /**
     * Information for the name column
     *
     * @param array $server a Server in array format
     *
     * @return string
     */
    public function column_name( $server )
    {
        $name = $server->name;
        $actions = $this->quickActionsForServer($server);

        return $name . $this->row_actions( $actions ) . $this->inlineData( $server );
    }

    public function inlineData($server)
    {
        $server_id = $server->id;
        $inline =<<<EOHTML
        <div id="inline-server-{$server_id}" class="hidden">
            <div class="name">{$server->name}</div>
        </div>
EOHTML;

        return $inline;
    }

    /**
     * Information for the server column
     *
     * @param array $server a Server in array format
     *
     * @return string
     */
    public function column_server( $server )
    {
        return sprintf('<span class="">%s</span>', $server->id);
    }

    /**
     * Information for the ip column
     *
     * @param array $server a Server in array format
     *
     * @return string
     */
    public function column_ip( $server )
    {
        return sprintf('<span class="">%s</span>', $server->lastaddress);
    }

    /**
     * Information for the registered column
     *
     * @param array $server a Server in array format
     *
     * @return string
     */
    public function column_registered( $server )
    {
        return sprintf('<span class="">%s</span>', true === $server->registered ? 'Yes' : 'No');
    }

    /**
     * Prepare the data for the table
     */
    public function prepare_items()
    {
        $this->_column_headers = $this->get_column_info();
        $this->process_bulk_action();

        $perPageOption = get_current_screen()->get_option('per_page');
        //$perPage    = $this->get_items_per_page( 'installs_per_page' , $perPageOption['default'] );
        $pageNumber = $this->get_pagenum();

        $platform_id = $_GET['platform'];
        $platform = $this->platformService->platform($platform_id);
        $servers = $platform->getServers();
        $platformServers = $this->platformServerService->getServersForPlatform($platform);
        foreach ($servers as $server) {
            $server->registered = false;
            foreach ($platformServers as $ps) {
                if ($server->id == $ps->getServerID()) {
                    $server->registered = true;
                    break; // when found
                }
            }
        }

        $this->set_pagination_args( [
            'total_items' => count($servers),
            //'per_page'    => $perPage
        ] );

        $options = ['asArray' => false];
        if (isset($_GET['orderby'])) {
            $options['orderby'] = $_GET['orderby'];
        }
        if (isset($_GET['order'])) {
            $options['order'] = $_GET['order'];
        }
        if (isset($_REQUEST['s'])) {
            $options['s'] = $_REQUEST['s'];
        }
        if (isset($_REQUEST['profile'])) {
            $options['profile'] = $_REQUEST['profile'];
        }

        $this->items = $servers;
        $this->stats['total'] = count($servers);
    }

    /**
     * Import multiple servers
     */
    public function action_bulk_import($ids)
    {
        foreach ( $ids as $id ) {
            $this->action_import(['server_id' => $id]);
        }

    }

    /**
     * Delete multiple servers
     */
    public function action_bulk_delete($ids)
    {
        foreach ( $ids as $id ) {
            $this->action_delete(['server_id' => $id]);
        }
    }

    /**
     * Import one server
     */
    public function action_import($args)
    {
        $platform_id = $_GET['platform'];
        $platform = $this->platformService->platform($platform_id);
        $servers = $platform->getServers();
        if (isset($args['server_id'])) {
            foreach ($servers as $server) {
                if ($server->id == $args['server_id']) {
                    // this is the server for import
                    $importServer = $server;
                    break;
                }
            }
            if (null === $importServer) {
                Helper::adminMessage('No server found for server_id:' . $args['server_id'], Helper::ERROR);
                return;
            }

            $knownServers = $this->platformServerService->getServersForPlatform($platform);
            $found = false;
            if (count($knownServers) > 0) {
                // found one or more servers registered to platform.
                foreach ($knownServers as $server) {
                    if ($server->getServerID() == $args['server_id']) {
                        $found = true;
                        Helper::adminMessage('Found existing server and updating: ' . $importServer->id, Helper::SUCCESS);
                        // found an existing server
                        // update its data with the latest
                        $server->setIP($importServer->lastaddress);
                        $server->setName($importServer->name);
                        $this->serverService->updateServer($server);
                    }
                }
            }
            if (false == $found) {
                // this is a new server
                Helper::adminMessage('Importing new server: ' . $importServer->id, Helper::SUCCESS);
                $newServer = $this->serverService->newModel();
                $newServer->setServerID($importServer->id);
                $newServer->setName($importServer->name);
                $newServer->setIP($importServer->lastaddress);
                $newServer = $this->serverService->createServer($newServer);
                $this->platformServerService->addServerToPlatform($newServer, $platform);
            }
        } else {
            Helper::adminMessage('Missing server_id for import', Helper::ERROR);
        }
    }

    /**
     * Delete one server
     */
    public function action_delete($args)
    {
        Helper::adminMessage('Delete is not yet implemented', Helper::ERROR);
    }

    public function getStats()
    {
        return $this->stats;
    }
}
