<?php declare(strict_types=1);
namespace ElLib;

/**
 * Library for working with the Elite License wordpress plugin, specifically for divimanager plugin and extensions
 * divimanager uses elite-licenser to manage its licensing
 *
 * Orginally by appsbd.com 
 * See https://appsbd.com/products/elite-licenser/
 */
class ElLib
{
    private $_onDeleteLicense = [];

    public function __construct($key, $productId, $productSlug, $licenseUrl, $licenseKey, $licenseEmail, $pluginBaseFile, $hasCheckUpdate = true, $isEncryptUpdate = true)
    {
        $this->key = $key;
        $this->productId = $productId;
        $this->productSlug = $productSlug;
        $this->licenseUrl = $licenseUrl;
        $this->licenseKey = $licenseKey;
        $this->licenseEmail = $licenseEmail;
        $this->pluginBaseFile = $pluginBaseFile;

        $this->hasCheckUpdate = $hasCheckUpdate;
        $this->isEncryptUpdate = $isEncryptUpdate;

        $this->version = $this->getCurrentVersion();

    }

    public function init()
    {
        if($this->hasCheckUpdate) {
            if(function_exists('add_action')){
                add_action( 'admin_post_'.$this->productSlug.'_fupc', [$this, 'adminPost'] );
                add_action( 'init', [$this, 'initActionHandler'] );

            }

            if(function_exists('add_filter')) {
                add_filter('pre_set_site_transient_update_plugins', [$this, 'PluginUpdate']);
                add_filter('plugins_api', [$this, 'checkUpdateInfo'], 10, 3);
                add_filter('plugin_row_meta', [$this, 'pluginMeta'], 10, 2);
                add_action('in_plugin_update_message-' . plugin_basename( $this->pluginBaseFile ), [$this,'updateMessageCB'], 20, 2 );
                add_action('upgrader_process_complete', [$this, 'upgraderProcessComplete'], 10, 2);
            }
        }
    }

    public function initActionHandler()
    {
        $handler = hash('crc32b', $this->productId . $this->key . $this->getDomain()).'_handle';
        if(isset($_GET['action']) && $_GET['action'] == $handler) {
            $this->handleServerRequest();
            exit;
        }
    }

    public function handleServerRequest()
    {
        $type=isset($_GET['type'])?strtolower($_GET['type']):"";
        switch ($type) {
            case "rl": //remove license
                $this->cleanUpdateInfo();
                $this->removeOldWPResponse();
                $obj          = new \stdClass();
                $obj->product = $this->productId;
                $obj->status  = true;
                echo $this->encryptObj( $obj );
                
                return;
            case "rc": //remove license
                $key  = $this->getKeyName();
                delete_option( $key );
                $obj          = new \stdClass();
                $obj->product = $this->productId;
                $obj->status  = true;
                echo $this->encryptObj( $obj );
                return;
            case "dl": //delete plugins
                $obj          = new \stdClass();
                $obj->product = $this->productId;
                $obj->status  = false;
                $this->removeOldWPResponse();
                require_once( ABSPATH . 'wp-admin/includes/file.php' );
                if ( $this->isTheme ) {
                    $res = delete_theme( $this->pluginBaseFile );
                    if ( ! is_wp_error( $res ) ) {
                        $obj->status = true;
                    }
                    echo $this->encryptObj( $obj );
                } else {
                    deactivate_plugins( [ plugin_basename( $this->pluginBaseFile ) ] );
                    $res = delete_plugins( [ plugin_basename( $this->pluginBaseFile ) ] );
                    if ( ! is_wp_error( $res ) ) {
                        $obj->status = true;
                    }
                    echo $this->encryptObj( $obj );
                }
                
                return;
            default:
                return;
        }
    }

    /**
     * @param callable $func
     */
    public function addOnDelete(callable $func)
    {
        $this->_onDeleteLicense[] = $func;
    }

    /**
     * Get the current version of the plugin
     *
     * @return int
     */
    public function getCurrentVersion()
    {
        if( !function_exists('get_plugin_data') ){
            require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
        }

        $data = get_plugin_data($this->pluginBaseFile);
        if (isset($data['Version'])) {
            return $data['Version'];
        }

        return 0;
    }

    /**
     * Get the domain of the site currently running
     */
    private function getDomain()
    {
        if (function_exists('site_url')) {
            return site_url();
        }

        if ( defined( 'WPINC' ) && function_exists( 'get_bloginfo' ) ) {
            return get_bloginfo( 'url' );
        } else {
            $base_url = ( ( isset( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] == 'on' ) ? 'https' : 'http' );
            $base_url .= '://' . $_SERVER['HTTP_HOST'];
            $base_url .= str_replace( basename( $_SERVER['SCRIPT_NAME'] ), '', $_SERVER['SCRIPT_NAME'] );

            return $base_url;
        }
    }

    /**
     * Clean up the transients from updates
     */
    public function cleanUpdateInfo()
    {
        update_option('_site_transient_update_plugins','');
        update_option('_site_transient_update_themes','');
        delete_transient($this->productSlug . '_up');
        set_site_transient('update_themes', null);
    }

    /**
     * Display a message in the plugin row on the plugins screen
     */
    public function updateMessageCB($data, $response)
    {
        if (is_array($data)) {
            $data = (object)$data;
        }
        if (isset($data->package) && empty($data->package)) {
            if (empty($data->update_denied_type)) {
                print  "<br/><span style='display: block; border-top: 1px solid #ccc;padding-top: 5px; margin-top: 10px;'>Please <strong>active product</strong> or  <strong>renew support period</strong> to get latest version</span>";
            } elseif ('L' == $data->update_denied_type) {
                print  "<br/><span style='display: block; border-top: 1px solid #ccc;padding-top: 5px; margin-top: 10px;'>Please <strong>active product</strong> to get latest version</span>";
            } elseif ('S' == $data->update_denied_type){
                print  "<br/><span style='display: block; border-top: 1px solid #ccc;padding-top: 5px; margin-top: 10px;'>Please <strong>renew support period</strong> to get latest version</span>";
            }
        }
    }

    protected function pluginUpdateInfo()
    {
        if(function_exists("wp_remote_get")) {
            $response = get_transient( $this->productSlug . '_up' );
            $oldFound = false;
            if ( ! empty( $response['data'] ) ) {
                $response = unserialize( $this->decrypt( $response['data'] ) );
                if ( is_array( $response ) ) {
                    $oldFound = true;
                }
            }

            if ( ! $oldFound ) {
                $licenseInfo = $this->GetRegisterInfo();
                $url = $this->licenseUrl . 'product/update/' . $this->productId;
                if (!empty($licenseInfo->license_key)) {
                    $url .= '/' . $licenseInfo->license_key . '/' . $this->version;
                }
                $args = [
                    'sslverify'   => true,
                    'timeout'     => 120,
                    'redirection' => 5,
                    'cookies'     => []
                ];
                $response = wp_remote_get( $url, $args );
                if (is_wp_error($response)) {
                    $args['sslverify'] = false;
                    $response = wp_remote_get( $url, $args );
                }
            }

            if (! is_wp_error($response)) {
                $body         = $response['body'];
                $responseJson = @json_decode( $body );
                if ( ! $oldFound ) {
                    set_transient( $this->productSlug . '_up', [
                        'data' => $this->encrypt( serialize( [ 'body' => $body ] ) )
                    ], DAY_IN_SECONDS );
                }

                if (!(is_object( $responseJson ) && isset($responseJson->status )) && $this->isEncryptUpdate) {
                    $body = $this->decrypt($body,$this->key);
                    $responseJson = json_decode( $body );
                }

                if ( is_object( $responseJson ) && ! empty( $responseJson->status ) && ! empty( $responseJson->data->new_version ) ) {
                    $responseJson->data->slug = plugin_basename( $this->pluginBaseFile );;
                    $responseJson->data->new_version = ! empty( $responseJson->data->new_version ) ? $responseJson->data->new_version : '';
                    $responseJson->data->url         = ! empty( $responseJson->data->url ) ? $responseJson->data->url : '';
                    $responseJson->data->package     = ! empty( $responseJson->data->download_link ) ? $responseJson->data->download_link : '';
                    $responseJson->data->update_denied_type     = ! empty( $responseJson->data->update_denied_type ) ? $responseJson->data->update_denied_type : '';

                    $responseJson->data->sections    = (array) $responseJson->data->sections;
                    $responseJson->data->plugin      = plugin_basename( $this->pluginBaseFile );
                    $responseJson->data->icons       = (array) $responseJson->data->icons;
                    $responseJson->data->banners     = (array) $responseJson->data->banners;
                    $responseJson->data->banners_rtl = (array) $responseJson->data->banners_rtl;
                    unset( $responseJson->data->IsStoppedUpdate );

                    return $responseJson->data;
                }
            }
        }

        return null;
    }

    /**
     * Before the transient value for the plugin is updated
     *
     * Unset the download link and isStoppedUpdate if there is an update and update the transient
     * Otherwise unset the transient
     */
    public function PluginUpdate($transient)
    {
        $response = $this->pluginUpdateInfo();

        if (!empty($response->plugin)) {
            $index_name = $response->plugin;
            if (!empty($response) && version_compare($this->version, $response->new_version, '<')) {
                unset($response->download_link);
                unset($response->IsStoppedUpdate);
                $transient->response[$index_name] = (object)$response;
            }else{
                if (isset($transient->response[$index_name])) {
                    unset($transient->response[$index_name]);
                }
            }
        }

        return $transient;
    }

    /**
     * Check for plugin update information
     */
    final function checkUpdateInfo($false, $action, $arg)
    {
        if ( empty($arg->slug)) {
            return $false;
        }

        if ( !empty($arg->slug) && $arg->slug === plugin_basename($this->pluginBaseFile) ) {
            $response = $this->pluginUpdateInfo();
            if ( !empty($response)) {
                return $response;
            }
        }

        return $false;
    }

    /**
     * Build a "renew license" link for the plugin
     */
    public function getRenewLink($responseObj, $type = "s")
    {
        if (empty($responseObj->renew_link)) {
            return '';
        }

        $isShowButton=false;
        if ('s' == $type) {
            $support_str=strtolower( trim( $responseObj->support_end ) );
            if ( strtolower( trim( $responseObj->support_end ) ) == "no support" ) {
                $isShowButton = true;
            } elseif ( !in_array($support_str, ["unlimited"] ) ) {
                if ( strtotime( 'ADD 30 DAYS', strtotime( $responseObj->support_end ) ) < time() ) {
                    $isShowButton = true;
                }
            }
            if ( $isShowButton ) {
                return $responseObj->renew_link.(strpos($responseObj->renew_link,"?")===FALSE?'?type=s&lic='.rawurlencode($responseObj->licenseKey):'&type=s&lic='.rawurlencode($responseObj->licenseKey));
            }
            return '';
        } else {
            $isShowButton=false;
            $expire_str=strtolower( trim( $responseObj->expire_date ) );
            if ( !in_array($expire_str, ["unlimited","no expiry"] )) {
                if ( strtotime( 'ADD 30 DAYS', strtotime( $responseObj->expire_date ) ) < time() ) {
                    $isShowButton = true;
                }
            }
            if ( $isShowButton ) {
                return $responseObj->renew_link.(strpos($responseObj->renew_link,"?")===FALSE?'?type=l&lic='.rawurlencode($responseObj->licenseKey):'&type=l&lic='.rawurlencode($responseObj->licenseKey));
            }
            return '';
        }
    }

    /**
     * Helpers for encrypting and decrypting requests and responses from the license server
     */
    public function encryptObj( $obj )
    {
        $text = serialize( $obj );

        return $this->encrypt( $text );
    }

    private function decryptObj( $ciphertext )
    {
        $text = $this->decrypt( $ciphertext );

        return unserialize( $text );
    }

    private function encrypt($plainText, $password = '')
    {
        if (empty($password)) {
            $password = $this->key;
        }
        $plainText = rand(10,99) . $plainText . rand(10,99);
        $method = 'aes-256-cbc';
        $key = substr( hash( 'sha256', $password, true ), 0, 32 );
        $iv = substr(strtoupper(md5($password)),0,16);
        return base64_encode( openssl_encrypt( $plainText, $method, $key, OPENSSL_RAW_DATA, $iv ) );
    }

    private function decrypt($encrypted, $password = '')
    {
        if (empty($password)) {
            $password = $this->key;
        }
        $method = 'aes-256-cbc';
        $key = substr( hash( 'sha256', $password, true ), 0, 32 );
        $iv = substr(strtoupper(md5($password)),0,16);
        $plaintext=openssl_decrypt( base64_decode( $encrypted ), $method, $key, OPENSSL_RAW_DATA, $iv );
        if (false === $plaintext) {
            // failed to decrypt, possibly old/stale value?
            return NULL;
        }
        return substr($plaintext,2,-2);
    }

    /**
     * Helpers for processing the response
     */
    private function processs_response($response)
    {
        $resbk = '';
        if ( ! empty( $response ) ) {
            if ( ! empty( $this->key ) ) {
                $resbk = $response;
                $response = $this->decrypt( $response );
            }
            $response = json_decode( $response );

            if ( is_object( $response ) ) {
                return $response;
            } else {
                $response         = new \stdClass();
                $response->status = false;
                $response->msg    = 'Response Error, contact the author or update the plugin';
                if (!empty($resbk)) {
                    $bkjson = @json_decode($resbk);
                    if (!empty($bkjson->msg)) {
                        $response->msg = $bkjson->msg;
                    }
                }
                $response->data = NULL;
                return $response;

            }
        }

        $response         = new \stdClass();
        $response->msg    = 'unknown response';
        $response->status = false;
        $response->data   = NULL;

        return $response;
    }

    /**
     * Make a request to the license server
     */
    private function _request( $relative_url, $data, &$error = '' )
    {
        $response                   = new \stdClass();
        $response->status           = false;
        $response->msg              = 'Empty Response';
        $response->is_request_error = false;
        $finalData                  = json_encode( $data );
        if ( ! empty( $this->key ) ) {
            $finalData = $this->encrypt( $finalData );
        }
        $url = rtrim( $this->licenseUrl, '/' ) . '/' . ltrim( $relative_url, '/' );
        if(function_exists('wp_remote_post')) {
            $rq_params=[
                'method' => 'POST',
                'sslverify' => true,
                'timeout' => 120,
                'redirection' => 5,
                'httpversion' => '1.0',
                'blocking' => true,
                'headers' => [],
                'body' => $finalData,
                'cookies' => []
            ];
            $serverResponse = wp_remote_post($url, $rq_params);

            if (is_wp_error($serverResponse)) {
                $rq_params['sslverify']=false;
                $serverResponse = wp_remote_post($url, $rq_params);
                 if (is_wp_error($serverResponse)) {
                    $response->msg    = $serverResponse->get_error_message();;
                    $response->status = false;
                    $response->data = NULL;
                    $response->is_request_error = true;
                    return $response;
                }else{
                     if(!empty($serverResponse['body']) && (is_array($serverResponse) && 200 === (int) wp_remote_retrieve_response_code( $serverResponse )) && $serverResponse['body']!='GET404' ){
                        return $this->processs_response($serverResponse['body']);
                    }
                }
            } else {
                if(!empty($serverResponse['body']) && (is_array($serverResponse) && 200 === (int) wp_remote_retrieve_response_code( $serverResponse )) && $serverResponse['body']!='GET404' ){
                    return $this->processs_response($serverResponse['body']);
                }
            }

        }
        if (!extension_loaded('curl')) {
            $response->msg    = 'Curl extension is missing';
            $response->status = false;
            $response->data = NULL;
            $response->is_request_error = true;
            return $response;
        }
        //curl when fall back
        $curlParams = [
            CURLOPT_URL            => $url,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_SSL_VERIFYPEER => true,
            CURLOPT_ENCODING       => '',
            CURLOPT_MAXREDIRS      => 10,
            CURLOPT_TIMEOUT        => 120,
            CURLOPT_CUSTOMREQUEST  => 'POST',
            CURLOPT_POSTFIELDS     => $finalData,
            CURLOPT_HTTPHEADER     => array(
                'Content-Type: text/plain',
                'cache-control: no-cache'
            )
        ];
        $curl = curl_init();
        curl_setopt_array( $curl, $curlParams);
        $serverResponse = curl_exec( $curl );
        $curlErrorNo=curl_errno($curl);
        $error = curl_error( $curl );
        curl_close( $curl );
        if (!$curlErrorNo) {
            if ( ! empty( $serverResponse ) ) {
                return $this->processs_response($serverResponse);
            }
        }else{
            $curl  = curl_init();
            $curlParams[CURLOPT_SSL_VERIFYPEER]=false;
            $curlParams[CURLOPT_SSL_VERIFYHOST]=false;
            curl_setopt_array( $curl, $curlParams);
            $serverResponse = curl_exec( $curl );
            $curlErrorNo=curl_errno($curl);
            $error = curl_error( $curl );
            curl_close( $curl );
            if(!$curlErrorNo){
                if ( ! empty( $serverResponse ) ) {
                    return $this->processs_response($serverResponse);
                }
            }else{
                $response->msg    = $error;
                $response->status = false;
                $response->data = NULL;
                $response->is_request_error = true;
                return $response;
            }
        }
        $response->msg    = 'unknown response';
        $response->status = false;
        $response->data = NULL;
        $response->is_request_error = true;
        return $response;
    }

    /**
     * Generate parameters for request
     */
    private function getParam( $licenseKey, $app_version, $admin_email = '' ) {
        $req               = new \stdClass();
        $req->license_key  = $licenseKey;
        $req->email        = ! empty( $admin_email ) ? $admin_email : $this->licenseEmail;
        $req->domain       = $this->getDomain();
        $req->app_version  = $app_version;
        $req->product_id   = $this->productId;
        $req->product_base = $this->productSlug;

        return $req;
    }

    private function getKeyName() {
        return hash( 'crc32b', $this->getDomain() . $this->pluginBaseFile . $this->productId . $this->productSlug . $this->key . 'LIC' );
    }

    /**
     * Store the response for later use
     */
    private function SaveWPResponse( $response )
    {
        $key  = $this->getKeyName();
        $data = $this->encrypt( serialize( $response ), $this->getDomain() );
        update_option( $key, $data );
    }

    /**
     * Retrieve the response from earlier
     */
    private function getOldWPResponse()
    {
        $key  = $this->getKeyName();
        $response = get_option( $key, NULL );
        if ( empty( $response ) ) {
            return NULL;
        }

        return unserialize( $this->decrypt( $response, $this->getDomain() ) );
    }

    /**
     * Remove the saved response
     * Calling any onDeleteLicense linked callables
     */
    private function removeOldWPResponse()
    {
        $key  = $this->getKeyName();
        $isDeleted = delete_option( $key );
        foreach ( $this->_onDeleteLicense as $func ) {
            if ( is_callable( $func ) ) {
                call_user_func( $func );
            }
        }

        return $isDeleted;
    }

    /**
     * Check withthe license server about this plugin
     */
    public function CheckWPPlugin(&$error = '', &$responseObj = null)
    {
        $restUrl = \add_query_arg([]);
        if (false === strpos($restUrl, '/?rest_route=%2Flicensor')) {
            $this->init();
            return $this->_CheckWPPlugin($error, $responseObj);
        }
    }

    public function GetRegisterInfo()
    {
        return $this->getOldWPResponse();
    }

    final function _CheckWPPlugin( &$error = "", &$responseObj = null )
    {
        if(empty($this->licenseKey)){
            $this->removeOldWPResponse();
            $error = '';
            return false;
        }
        $oldRespons=$this->getOldWPResponse();
        $isForce=false;
        if(!empty($oldRespons)) {
            if ( ! empty( $oldRespons->expire_date ) && strtolower( $oldRespons->expire_date ) != "no expiry" && strtotime( $oldRespons->expire_date ) < time() ) {
                $isForce = true;
            }
            if ( ! $isForce && ! empty( $oldRespons->is_valid ) && $oldRespons->next_request > time() && ( ! empty( $oldRespons->license_key ) && $this->licenseKey == $oldRespons->license_key ) ) {
                $responseObj = clone $oldRespons;
                unset( $responseObj->next_request );

                return true;
            }
        }

        $param    = $this->getParam( $this->licenseKey, $this->version );
        $response = $this->_request( 'product/active/'.$this->productId, $param, $error );
        if(empty($response->is_request_error)) {
            if ( empty( $response->code ) ) {
                if ( ! empty( $response->status ) ) {
                    if ( ! empty( $response->data ) ) {
                        $serialObj = $this->decrypt( $response->data, $param->domain );

                        $licenseObj = unserialize( $serialObj );
                        if ( $licenseObj->is_valid ) {
                            $responseObj           = new \stdClass();
                            $responseObj->is_valid = $licenseObj->is_valid;
                            if ( $licenseObj->request_duration > 0 ) {
                                $responseObj->next_request = strtotime( "+ {$licenseObj->request_duration} hour" );
                            } else {
                                $responseObj->next_request = time();
                            }
                            $responseObj->expire_date   = $licenseObj->expire_date;
                            $responseObj->support_end   = $licenseObj->support_end;
                            $responseObj->license_title = $licenseObj->license_title;
                            $responseObj->license_key   = $this->licenseKey;
                            $responseObj->msg           = $response->msg;
                            $responseObj->renew_link           = !empty($licenseObj->renew_link)?$licenseObj->renew_link:"";
                            $responseObj->expire_renew_link           = $this->getRenewLink($responseObj,"l");
                            $responseObj->support_renew_link           = $this->getRenewLink($responseObj,"s");
                            $this->SaveWPResponse( $responseObj );
                            unset( $responseObj->next_request );
                            delete_transient($this->productSlug."_up");
                            return true;
                        } else {
                            if ( $this->__checkoldtied( $oldRespons, $responseObj, $response ) ) {
                                return true;
                            } else {
                                $this->removeOldWPResponse();
                                $error = ! empty( $response->msg ) ? $response->msg : "";
                            }
                        }
                    } else {
                        $error = "Invalid data";
                    }

                } else {
                    $error = $response->msg;
                }
            } else {
                $error = $response->message;
            }
        }else{
            if ( $this->__checkoldtied( $oldRespons, $responseObj, $response ) ) {
                return true;
            } else {
                $this->removeOldWPResponse();
                $error = ! empty( $response->msg ) ? $response->msg : "";
            }
        }
        return $this->__checkoldtied($oldRespons,$responseObj);
    }

    private function __checkoldtied(&$oldRespons,&$responseObj)
    {
        if(!empty($oldRespons) && (empty($oldRespons->tried) || $oldRespons->tried<=2)){
            $oldRespons->next_request = strtotime("+ 1 hour");
            $oldRespons->tried=empty($oldRespons->tried)?1:($oldRespons->tried+1);
            $responseObj = clone $oldRespons;
            unset( $responseObj->next_request );
            if(isset($responseObj->tried)) {
                unset( $responseObj->tried );
            }
            $this->SaveWPResponse($oldRespons);
            return true;
        }
        return false;
    }

    public function adminPost()
    {
        $this->cleanUpdateInfo();
        wp_redirect(  admin_url( 'plugins.php' ) );
        exit;
    }

    /**
     * Add link to the row for this plugin
     */
    public function pluginMeta($links, $plugin_file)
    {
        if ( $plugin_file == plugin_basename( $this->pluginBaseFile ) ) {
            $links[] = " <a class='edit coption' href='" . esc_url( admin_url( 'admin-post.php' ) . '?action=' . $this->productSlug . '_fupc' ) . "'>Update Check</a>";
        }
        return $links;
    }

    /**
     * Do cleanup when the upgrader process is complete
     */
    public function upgraderProcessComplete($upgrader_object, $options)
    {
        $this->cleanUpdateInfo();
    }
}
