<?php

namespace ServerPilotAPI\Request;

use GuzzleHttp\Client;

class Site extends AbstractRequest
{
    public $endpoint = '/v1/apps';

    /**
     * Example response
     *
     * "id": "B1w7yc1tfUPQLIKS",
     * "datecreated": 1403143012,
     * "name": "store",
     * "sysuserid": "RvnwAIfuENyjUVnl",
     * "domains": ["www.mystore.com", "mystore.com"],
     * "ssl": {
     *   "key": "-----BEGIN PRIVATE KEY----- ...",
     *   "cert": "-----BEGIN CERTIFICATE----- ...",
     *   "cacerts": "-----BEGIN CERTIFICATE----- ...",
     *   "auto": false,
     *   "force": false
     * },
     * "serverid": "4zGDDO2xg30yEeum",
     * "runtime": "php7.0"
     */
    public function getList()
    {
        $response = $this->client->request('GET', $this->endpoint, [
            'auth' => [$this->clientID, $this->apiKey]
        ]);

        $body = $response->getBody();
        $data = json_decode($body)->data;
        $sites = [];
        foreach ($data as $site) {
            $autossl = isset($site->ssl) && isset($site->ssl->auto) ? $site->ssl->auto : false;
            $forcessl = isset($site->ssl) && isset($site->ssl->force) ? $site->ssl->force : false;
            $sites[] = [
                'id' => $site->id,
                'name' => $site->name,
                'domains' => $site->domains,
                'dateCreated' => $site->datecreated,
                'sysuserid' => $site->sysuserid,
                'ssl' => [
                    'auto' => $autossl,
                    'force' => $forcessl
                ],
                'serverid' => $site->serverid,
                'runtime' => $site->runtime
            ];
        }

        return $sites;
    }

    public function get($id)
    {
        $response = $this->client->request('GET', $this->endpoint . '/' . $id, [
            'auth' => [$this->clientID, $this->apiKey],
        ]);

        $site = null;
        $body = $response->getBody();
        $site = json_decode($body)->data;
        $autossl = isset($site->ssl) && isset($site->ssl->auto) ? $site->ssl->auto : false;
        $forcessl = isset($site->ssl) && isset($site->ssl->force) ? $site->ssl->force : false;
        $data = [
            'id' => $site->id,
            'name' => $site->name,
            'domains' => $site->domains,
            'dateCreated' => $site->datecreated,
            'sysuserid' => $site->sysuserid,
            'ssl' => [
                'auto' => $autossl,
                'force' => $forcessl
            ],
            'serverid' => $site->serverid,
            'runtime' => $site->runtime
        ];
        return $data;
    }

    /**
     * Create a new app on a server
     * wordpress setup by default
     * runtime php7.4
     * user and server must both be created ahead of time
     */
    public function create($name, $user, $domain, $runtime = 'php7.4')
    {
        if (! is_array($domain)) {
            $domain = [$domain];
        }

        $response = $this->client->post($this->endpoint, [
            'auth' => [$this->clientID, $this->apiKey],
            'json' => [
                'name' => $name,
                'sysuserid' => $user,
                'runtime' => $runtime,
                'domains' => $domain,
                'wordpress' => [
                    'site_title' => $name,
                    'admin_user' => $this->adminUser,
                    'admin_password' => bin2hex(random_bytes(10)),
                    'admin_email' => $this->adminEmail,
                ],
            ]
        ]);

        return json_decode($response->getBody());
    }

    /**
     * Enable autossl
     *
     * enable ssl by default
     */
    public function autossl($site, $auto = true)
    {
        $response = $this->client->post($this->endpoint . '/' . $site . '/ssl', [
            'auth' => [$this->clientID, $this->apiKey],
            'json' => [
                'auto' => $auto,
            ],
        ]);

        return json_decode($response->getBody());
    }

    /**
     * Enable force-ssl
     *
     * enable force-ssl by default
     */
    public function forcessl($site, $auto = true)
    {
        $response = $this->client->post($this->endpoint . '/' . $site . '/ssl', [
            'auth' => [$this->clientID, $this->apiKey],
            'json' => [
                'force' => $auto,
            ],
        ]);

        return json_decode($response->getBody());
    }

    /**
     * Update domains for site
     */
    public function updateDomains($site, $domains)
    {
        $response = $this->client->post($this->endpoint . '/' . $site, [
            'auth' => [$this->clientID, $this->apiKey],
            'json' => [
                'domains' => $domains,
            ],
        ]);

        return json_decode($response->getBody());
    }

    public function setAdminUser($adminUser)
    {
        $this->adminUser = $adminUser;
    }
    public function setAdminEmail($adminEmail)
    {
        $this->adminEmail = $adminEmail;
    }
    public function setPhpRuntime($phpRuntime)
    {
        $this->phpRuntime = $phpRuntime;
    }
}
