<?php

declare(strict_types=1);

namespace ServerPilotAPI\Request;

use PHPUnit\Framework\TestCase;
use GuzzleHttp\Client;
use GuzzleHttp\Handler\MockHandler;
use GuzzleHttp\HandlerStack;
use GuzzleHttp\Psr7\Response;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Exception\RequestException;

final class SiteTest extends TestCase
{
    public function setUp(): void
    {
        /**
         * These are example data sets from the serverpilot api documentation
         */
        $this->singleSiteResponse = (object) [
          'id' => 'B1w7yc1tfUPQLIKS',
          'datecreated' => 1403143012,
          'name' => 'store',
          'sysuserid' => 'RvnwAIfuENyjUVnl',
          'domains' => ['www.mystore.com', 'mystore.com'],
          'ssl' => (object) [
            'key' => '-----BEGIN PRIVATE KEY----- ...',
            'cert' => '-----BEGIN CERTIFICATE----- ...',
            'cacerts' => '-----BEGIN CERTIFICATE----- ...',
            'auto' => false,
            'force' => false
          ],
          'serverid' => '4zGDDO2xg30yEeum',
          'runtime' => 'php7.0'
        ];

        /**
         * These are the expected results given the above data
         */
        $this->singleSiteData = [
                'id' => 'B1w7yc1tfUPQLIKS',
                'name' => 'store',
                'domains' => [
                    'www.mystore.com',
                    'mystore.com'
                ],
                'dateCreated' => 1403143012,
                'sysuserid' => 'RvnwAIfuENyjUVnl',
                'ssl' => [
                    'auto' => false,
                    'force' => false
                ],
                'serverid' => '4zGDDO2xg30yEeum',
                'runtime' => 'php7.0',

        ];
    }

    public function testGetList(): void
    {
        $responseData = [
            'data' =>  [$this->singleSiteResponse],
        ];
        $response = json_encode($responseData);
        $mock = new MockHandler([
            new Response(200, ['X-Foo' => 'Bar'], $response),
        ]);

        $handlerStack = HandlerStack::create($mock);
        $client = new Client(['handler' => $handlerStack]);
        $url = 'foo';
        $sut = new Site($client, $url);
        $response = $sut->getList();
        $this->assertCount(1, $response);
        $this->assertEquals(
            [$this->singleSiteData],
            $response
        );
    }

    public function testGet(): void
    {
        $responseData = [
            'data' =>  $this->singleSiteResponse,
        ];
        $response = json_encode($responseData);
        $mock = new MockHandler([
            new Response(200, ['X-Foo' => 'Bar'], $response),
        ]);

        $handlerStack = HandlerStack::create($mock);
        $client = new Client(['handler' => $handlerStack]);
        $url = 'foo';
        $sut = new Site($client, $url);
        $response = $sut->get(1);
        $this->assertEquals(
            $this->singleSiteData,
            $response
        );
    }

    public function testAutoSsl(): void
    {
        $expected = (object) [
           'actionid' => 'pneQJGaAwIyIojW8',
           'data' => (object) [
              'key' => '-----BEGIN PRIVATE KEY----- ... -----END PRIVATE KEY-----',
              'cert' => '-----BEGIN CERTIFICATE----- ... -----END CERTIFICATE-----',
              'cacerts' => '-----BEGIN CERTIFICATE----- ... -----END CERTIFICATE-----',
              'auto' => true,
              'force' => false
           ]
        ];
        $responseData = ['data' => $this->singleSiteResponse];
        $response = json_encode($expected);
        $mock = new MockHandler([
            new Response(200, ['X-Foo' => 'Bar'], $response),
        ]);

        $handlerStack = HandlerStack::create($mock);
        $client = new Client(['handler' => $handlerStack]);
        $url = 'foo';
        $sut = new Site($client, $url);
        $response = $sut->autossl(1);
        $this->assertEquals(
            $expected,
            $response
        );
    }

    public function testForceSsl(): void
    {
        $expected = (object) [
           'actionid' => 'pneQJGaAwIyIojW8',
           'data' => (object) [
              'key' => '-----BEGIN PRIVATE KEY----- ... -----END PRIVATE KEY-----',
              'cert' => '-----BEGIN CERTIFICATE----- ... -----END CERTIFICATE-----',
              'cacerts' => '-----BEGIN CERTIFICATE----- ... -----END CERTIFICATE-----',
              'auto' => true,
              'force' => true
           ]
        ];
        $responseData = ['data' => $this->singleSiteResponse];
        $response = json_encode($expected);
        $mock = new MockHandler([
            new Response(200, ['X-Foo' => 'Bar'], $response),
        ]);

        $handlerStack = HandlerStack::create($mock);
        $client = new Client(['handler' => $handlerStack]);
        $url = 'foo';
        $sut = new Site($client, $url);
        $response = $sut->forcessl(1);
        $this->assertEquals(
            $expected,
            $response
        );
    }

    public function testDomains(): void
    {
        $expected = (object) [
            'actionid' => 'KlsNzLikw3BRvShc',
            'data' => (object) [
                'id' => 'nlcN0TwdZAyNEgdp',
                'datecreated' => 1403143012,
                'name' => 'gallery',
                'sysuserid' => 'RvnwAIfuENyjUVnl',
                'domains' => ['foo.bar'],
                'ssl' => null,
                'serverid' => '4zGDDO2xg30yEeum',
                'runtime' => 'php5.6'
            ]
        ];
        $response = json_encode($expected);
        $mock = new MockHandler([
            new Response(200, ['X-Foo' => 'Bar'], $response),
        ]);

        $handlerStack = HandlerStack::create($mock);
        $client = new Client(['handler' => $handlerStack]);
        $url = 'foo';
        $sut = new Site($client, $url);
        $domains = ['foo.bar'];
        $response = $sut->updateDomains(1, $domains);
        $this->assertEquals(
            $expected,
            $response
        );
    }
}
